/** @file entrymenu.cpp
 *  @brief Gg[j[NXCvg
 *
 *  @author yasufuku
 *  @date 2011/01
 *  @version 0.10
 *
 *  Copyright(C) 2011 Kensuke Yasufuku
 */
#include "entrymenu.h"
#include "view.h"
#include "keyboard.h"
#include "samp_base.h"
#include "walk_sim.h"

static const float SIZE_X = 12;
static const float SIZE_Y = 24;

static MenuItem menu_item[] =
{
	//{ "Motion Capture", SampBase::Create, NULL },
	{ "Walkthrough -Villa Savoye-", WalkSim::Create, NULL },
	{ NULL, NULL, NULL }
};

//---------------------------------------------------------------
// 
//---------------------------------------------------------------
Task * EntryMenu::Create( Task * pParent, int nPriority )
{
	EntryMenu * p = NEW EntryMenu;
	if(p) p->Init( pParent, nPriority );
	return (Task *)p;
}

//---------------------------------------------------------------
// RXgN^
//---------------------------------------------------------------
EntryMenu::EntryMenu()
{
	SetName("Personal Game Engine DEMO");
	m_pScene       = NULL;
	m_pMenuFont    = NULL;
	m_pInfo        = NULL;
	m_nMenu        = 0;
	m_nCurrentMenu = 0;
	m_bDispMenu    = true;
	m_pBackground  = NULL;

	m_nEnter  = 0;
	m_nTimer1 = 0;
	m_nTimer2 = 0;
}

//---------------------------------------------------------------
// fXgN^
//---------------------------------------------------------------
EntryMenu::~EntryMenu()
{
}

//---------------------------------------------------------------
// 
//---------------------------------------------------------------
int EntryMenu::OnInit()
{
	int i;
	
	// Scene
	m_pScene = Scene::Create( View::GetRoot() );

	// 2DJ
	Camera::Create( m_pScene );

	m_nMenu=0;
	for( i=0; menu_item[i].pszName; i++ ) {
		m_nMenu++;
	}
	if( m_nMenu == 0 ) return 0;

	// wi
	m_pBackground = Sprite::Create( m_pScene, TEX_DIR"background.tga", 1280.0f, 720.0f );


	// tHg
	m_pMenuFont = Font::Create( m_pScene );
	m_pMenuFont->SetSize( 15, 30 );
	m_pMenuFont->SetColor( Color( 0.5f, 0.1f, 0.1f ) );
	m_pMenuFont->Print( -620, -50, "Personal Game Engine DEMO" );

	Font * pItem;
	for( i=0; i<m_nMenu; i++ ) {
		pItem = Font::Create( m_pMenuFont );
		pItem->ConnectParent(true);
		pItem->SetColor( Color( 0.5f, 0.5f, 0.5f ) );
		pItem->SetSize(SIZE_X, SIZE_Y);
		pItem->Print( 40.0f, SIZE_Y*2 + i*SIZE_Y + 4.0f, menu_item[i].pszName );
	}
	((Font *)m_pMenuFont->Child(m_nCurrentMenu))->SetColor( Color( 0.3f, 0.3f, 0.3f ) );

	m_pInfo = Font::Create( pItem );
	m_pInfo->SetColor( Color( 0.4f, 0.4f, 0.4f ) );
	m_pInfo->SetSize(8, 16);
	m_pInfo->Print( 0.0f, SIZE_Y, "Press Enter Key !" );	


	return TR_OK;
}

//---------------------------------------------------------------
// C
//---------------------------------------------------------------
int EntryMenu::OnMain()
{
	if( m_bDispMenu && Keyboard::Trigger(KEY_UP) ) {
		((Font *)m_pMenuFont->Child(m_nCurrentMenu))->SetColor( Color( 0.5f, 0.5f, 0.5f ) );
		if( m_nCurrentMenu > 0 ) {
			m_nCurrentMenu--;
		}
		((Font *)m_pMenuFont->Child(m_nCurrentMenu))->SetColor( Color( 0.3f, 0.3f, 0.3f ) );
	}
	if( m_bDispMenu && Keyboard::Trigger(KEY_DOWN) ) {
		((Font *)m_pMenuFont->Child(m_nCurrentMenu))->SetColor( Color( 0.5f, 0.5f, 0.5f ) );
		if( m_nCurrentMenu < m_nMenu-1 ) {
			m_nCurrentMenu++;
		} else {
			m_nCurrentMenu=0;
		}
		((Font *)m_pMenuFont->Child(m_nCurrentMenu))->SetColor( Color( 0.3f, 0.3f, 0.3f ) );
	}
	
	
	// Loadʏô1t[x点
	if( m_nEnter ) {
		if( m_pScene ) {
			m_pScene->Destroy();
			m_pScene = NULL;
		}
		(*menu_item[m_nCurrentMenu].CreateTaskFunc)( this );
		m_nEnter=0;
	}
	if( m_bDispMenu && (Keyboard::Trigger(KEY_A)|Keyboard::Trigger(KEY_ENTER)) ) {
		m_pInfo->Print( 0.0f, SIZE_Y, "Now Loading..." );
		m_pInfo->SetColor( Color( 0.5f, 0.0f, 0.0f ) );
		m_nEnter = 1;
		m_bDispMenu = false;
	}

	if(  Keyboard::Trigger(KEY_B) | Keyboard::Trigger(KEY_ESC) ) {
		if( ! m_bDispMenu ) {
			DestroyAllChildren();
			OnInit();
			m_bDispMenu = true;
		} else {
			if( Keyboard::Trigger(KEY_ESC) ) {
				Destroy();
				exit(0);
			}
		}
	}

	if( m_bDispMenu ) {
		m_nTimer1++; m_nTimer2++;

		if( m_nTimer1 < 60 ) m_pMenuFont->SetAlpha( (float)m_nTimer1/60.0f );
		else                 m_pMenuFont->SetAlpha( 1.0f );

		float alpha = 0.5f*(1.0f+sin( DegToRad( (float)(m_nTimer2*4) ) ));
		m_pInfo->SetAlpha(alpha);
	}


	return TR_OK;
}

//---------------------------------------------------------------
// 
//---------------------------------------------------------------
int EntryMenu::OnKill()
{
	if( m_pScene ) {
		m_pScene->Destroy();
		m_pScene = NULL;
	}
	return TR_OK;
}
