/** @file samp_base.cpp
 *  @brief PGETvx[XNXCvg
 *
 *  @author yasufuku
 *  @date 2011/01
 *  @version 0.10
 *
 *  Copyright(C) 2011 Kensuke Yasufuku
 */
#include "samp_base.h"
#include "mouse.h"
#include "keyboard.h"
#include "primitive.h"
#include "sprite.h"
#include "font.h"

// Of[^
extern ModelData  box_model;
extern ModelData  sphere_model;
extern ModelData  lady_model;
extern ModelData  savoye_model;
extern ModelData  skydome_model;

extern MotionData lady_dance_motion;

#define MODEL_NUM	   1


//---------------------------------------------------------------
//	eNX`C[We[u
//---------------------------------------------------------------
static TexImage font_tex_image[] =
{
	{
		TEX_DIR"background.tga",
		720,
		1280,
		GL_RGBA,
	},
};

//---------------------------------------------------------------
//	eNX`wb_e[u
//---------------------------------------------------------------
static TexHeader font_tex_header =
{
	0,
	font_tex_image,
	GL_REPEAT,
	GL_REPEAT,
	GL_NEAREST,
	GL_NEAREST,
};

//---------------------------------------------------------------
// 
//---------------------------------------------------------------
Task * SampBase::Create( Task * pParent )
{
	SampBase * p = NEW SampBase;
	p->Init( pParent, 0 );
	return (Task *)p;
}

//---------------------------------------------------------------
// RXgN^
//---------------------------------------------------------------
SampBase::SampBase()
{
	SetName("SampBase");

	m_fDist  = 12.0f;
	m_fTheta = 100.0f;
	m_fPhi   = -5.0f;
	m_fParam1 = 0.0f;

	m_pScene  = NULL;
	m_pCamera = NULL;
	m_pLight  = NULL;

	m_pBuilding = NULL;
	m_pSkyDome  = NULL;
	m_unModel = MODEL_NUM;
	m_ppModel  = NULL;

	m_pLine = NULL;
}

//---------------------------------------------------------------
// fXgN^
//---------------------------------------------------------------
SampBase::~SampBase()
{
}

//---------------------------------------------------------------
// 
//---------------------------------------------------------------
int SampBase::OnInit()
{
	//--------------------------------------------------------
	// 3DV[
	//--------------------------------------------------------
	m_pScene = Scene::Create( View::GetRoot() );

	//--------------------------------------------------------
	// J
	//--------------------------------------------------------
	Vector pos, at;
	
	at.x  = 0.0f;//7.3f;
	at.y  = 0.0f;//1.74f;
	at.z  = 0.0f;//-9.8f;

	float distxz = m_fDist * cos( DegToRad(m_fPhi) );
	pos.x = distxz * cos( DegToRad(m_fTheta) ) + at.x;
	pos.y = m_fDist* sin( DegToRad(m_fPhi  ) ) + at.y;
	pos.z = distxz * sin( DegToRad(m_fTheta) ) + at.z;

	m_pCamera = Camera::Create( m_pScene, pos, at );
	//m_pCamera->SetFovy(90.0f);
	
	//--------------------------------------------------------
	// Cg
	//--------------------------------------------------------
	m_vLightDir.Set( 5.0f*cos(m_fParam1), 4.0f, 5.0f*sin(m_fParam1) );
	m_pLight = Light::Create( m_pScene );
	m_pLight->SetDirection( m_vLightDir );
	m_pLight->Intensity( Color(1.0f, 0.0f, 0.0f) );
	
	//--------------------------------------------------------
	// f
	//--------------------------------------------------------
	m_pSkyDome  = Model::Create( m_pScene, &skydome_model );
#if 1
	m_ppModel = (Model **)NEW u32[m_unModel];

	for( int i=0; i<(int)m_unModel; i++ ) {
		m_ppModel[i] = Model::Create( m_pScene, &lady_model );
		m_ppModel[i]->t.x = (1.6f+Random(0.05f))* (float)(i%12) - 2.0f;
		m_ppModel[i]->t.z =-(1.6f+Random(0.05f))* (float)(i/12);
		m_ppModel[i]->SetMotion( &lady_dance_motion );
	}
#endif
	//m_pBuilding = Model::Create( m_pScene, &savoye_model );
	//m_pBuilding->r.z = PI*0.5f;
	
	return TR_OK;

}

//---------------------------------------------------------------
// 
//---------------------------------------------------------------
int SampBase::OnKill()
{
	m_pScene->Destroy();
	delete [] m_ppModel;
	return TR_OK;
}

//---------------------------------------------------------------
// C
//---------------------------------------------------------------
int SampBase::OnMain()
{
	for( int i=0; i<(int)m_unModel; i++ ) {
		if( Keyboard::Press(KEY_RIGHT)) m_ppModel[i]->IncTime(0.45f+Random(0.1f));
		if( Keyboard::Press(KEY_LEFT )) m_ppModel[i]->IncTime(-0.45f-Random(0.1f));
	}


	m_fParam1 += 0.01f;
	m_vLightDir.Set( 5.0f*cos(m_fParam1), 4.0f, 5.0f*sin(m_fParam1) );
	m_pLight->SetDirection( m_vLightDir );

#if 1
	//--------------------------------------------------------
	// J
	//--------------------------------------------------------
	int vx, vy;
	if( Mouse::Drag( MOUSE_L_BUTTON ) ) {
		Mouse::GetDrag( vx, vy );
		m_fTheta += (float)vx * 0.5f;
		if( vy > 0.0f ) {
			if( m_fPhi <  80.0 ) m_fPhi += (float)vy*0.2f;
		} else {
			if( m_fPhi > -80.0 ) m_fPhi += (float)vy*0.2f;
		}
	}

	if( Mouse::Drag( MOUSE_M_BUTTON ) ) {
		Mouse::GetDrag( vx, vy );
		m_fDist += (float)vx * -0.03f;		
		if( m_fDist < 0.1f ) {
			m_fDist = 0.1f;
		}
	}

	if( Mouse::Drag( MOUSE_R_BUTTON ) ) {
		Mouse::GetDrag( vx, vy );
		m_pCamera->lookat.x += (float)vx * cos( DegToRad(m_fTheta+90) ) * 0.02f;
		m_pCamera->lookat.z += (float)vx * sin( DegToRad(m_fTheta+90) ) * 0.02f;
		m_pCamera->lookat.y += (float)vy*0.03f;
	}

	float distxz = m_fDist * cos( DegToRad(m_fPhi) );	
	m_pCamera->t.x = distxz * cos( DegToRad(m_fTheta) ) + m_pCamera->lookat.x;
	m_pCamera->t.y = m_fDist* sin( DegToRad(m_fPhi  ) ) + m_pCamera->lookat.y;
	m_pCamera->t.z = distxz * sin( DegToRad(m_fTheta) ) + m_pCamera->lookat.z;

	//printf("Camera Pos (%f,%f,%f)\n", m_pCamera->t.x, m_pCamera->t.y, m_pCamera->t.z);
	//printf("Camera Look(%f,%f,%f)\n", m_pCamera->lookat.x, m_pCamera->lookat.y, m_pCamera->lookat.z);
	//printf("Camera Para(%f,%f,%f)\n", m_fDist, m_fTheta, m_fPhi);
#endif

	return TR_OK;
}
