/** @file element.cpp
 *  @brief ElementNXCvg
 *
 *  @author yasufuku
 *  @date 2011/02
 *  @version 0.20
 *
 *  Copyright(C) 2004-2011 Kensuke Yasufuku
 */
//--------------------------------------------------------------------
//	V[̍\vfƂȂ̂́AElementNXh
//
//		Element ---> Scene
//		        +--> Camera
//              +--> Light
//              +--> Model			etc...
//--------------------------------------------------------------------
//====================================================================
// CN[h
//====================================================================
#include "element.h"

//====================================================================
// ElementNXo֐`
//====================================================================
//--------------------------------------------------------------
// 
//--------------------------------------------------------------
/**
 *	@param pParent		eGg
 *	@param nPriority	Dx
 */
Element * Element::Create( Element * pParent, int nPriority )
{
	Element * pElm = NEW Element;
	return (Element *)pElm->Init( pParent, nPriority );
}

//--------------------------------------------------------------
// RXgN^
//--------------------------------------------------------------
Element::Element()
{
	SetName("Element");
	m_bDisp = true;
	m_unFlag = 0;
	m_Color.Set( 1.0f, 1.0f, 1.0f, 1.0f );

	t.Set();
	r.Set();
	s.Set(1.0f, 1.0f, 1.0f);
}

//--------------------------------------------------------------
// fXgN^
//--------------------------------------------------------------
Element::~Element()
{
}

//--------------------------------------------------------------
// bZ[WfBXpb`
//--------------------------------------------------------------
int Element::Receive( TaskMessage * pMessage )
{
	int nResult;

	switch( pMessage->Tag() )
	{
	  case TM_INIT:
		nResult = OnInit();
		break;
		
	  case TM_MAIN:
		nResult = OnMain();
		break;

	  case TM_KILL:
		nResult = OnKill();
		break;

	  case TM_UPDATE:
		nResult = OnUpdate();
		break;

	  case TM_RENDER:
		if( m_bDisp ) { nResult = OnRender(); }
		else nResult = TR_OK;
		break;

	  case TM_RESIZE:
		nResult = OnResize();
		break;
		
	  default:
	    nResult = TR_ERROR;
		assert(false);
		break;
	}
	return nResult;
}

//--------------------------------------------------------------
// 
//--------------------------------------------------------------
int Element::OnInit()
{
	return TR_OK;
}

//--------------------------------------------------------------
// C
//--------------------------------------------------------------
int Element::OnMain()
{
	return TR_OK;
}

//--------------------------------------------------------------
// XV
//--------------------------------------------------------------
int Element::OnUpdate()
{
	// }gbNXXV
	m.Unit();
	m.RotZYX( r );
	m.TransL( t.x, t.y, t.z );
	m.ScaleR( s );
	
	OnUpdateChildren();
	
	return TR_OK;
}

//--------------------------------------------------------------
// _O
//--------------------------------------------------------------
int Element::OnRender()
{
	return TR_OK;
}

//--------------------------------------------------------------
// EBhETCỸTCY
//--------------------------------------------------------------
int Element::OnResize()
{
	return TR_OK;
}

//--------------------------------------------------------------
// qGg̍XV
//--------------------------------------------------------------
int Element::OnUpdateChildren()
{
	// ^XNs͎q^XNɂ
	TaskList::iterator it = m_ChildList.begin();
	
	while( it != m_ChildList.end() ) {
		(*it)->Call( g_TMUpdate );
		it++;
	}
	return TR_OK;
}

//--------------------------------------------------------------
// GgXg\
//--------------------------------------------------------------
void Element::DispElementList()
{
	PRINTF("--------------------------------\n");
	PRINTF("  Element Tree\n");
	PRINTF("--------------------------------\n");

	DispChildren();

	PRINTF("--------------------------------\n");
}
