/** @file font.cpp
 *  @brief FontNXCvg
 *
 *  @author yasufuku
 *  @date 2004/12
 *  @version 0.10
 *
 *  Copyright(C) 2004 Kensuke Yasufuku
 */
#include "font.h"
#include "scene.h"
#include "texture_manager.h"
#include "shader_manager.h"
#include "sys_config.h"

extern SystemConfig sc;

namespace pge {
	
static Vector g_ParentPos;

//===============================================================
// tHgf[^
//===============================================================
//---------------------------------------------------------------
//	eNX`C[We[u
//---------------------------------------------------------------
static TexImage font_tex_image[] =
{
	{
		"../pge/data/tex/ascii.tga",
		192,
		192,
		GL_ALPHA,
	},
};

//---------------------------------------------------------------
//	eNX`wb_e[u
//---------------------------------------------------------------
static TexHeader font_tex_header =
{
	0,
	font_tex_image,
	GL_REPEAT,
	GL_REPEAT,
	GL_LINEAR,
	GL_LINEAR,
};

//===============================================================
// static
//===============================================================
//===============================================================
// FontNXo֐
//===============================================================
//---------------------------------------------------------------
// 
//---------------------------------------------------------------
Font * Font::Create( Element * pParent )
{
	Font * pFont = NEW Font();
	return (Font *)pFont->Init( pParent, TP_FONT+1 );
}

//---------------------------------------------------------------
// RXgN^
//---------------------------------------------------------------
Font::Font()
{
	SetName("Font");
	
	m_fWidth  = FONT_TEX_LETTER_X;
	m_fHeight = FONT_TEX_LETTER_Y;
	m_Color.Set( 1.0f, 1.0f, 1.0f, 1.0f );
	m_pszStr = "\0";
	m_ParentPos.Set( 0.0f, 0.0f, 0.0f );
	m_Pos.Set( 0.0f, 0.0f, 0.0f );
	m_bShadow = false;

	m_pShader  = NULL;
	m_paVAO    = NULL;
	m_paVBO    = NULL;
}

//---------------------------------------------------------------
// fXgN^
//---------------------------------------------------------------
Font::~Font()
{
	glDeleteVertexArrays( 1, m_paVAO );
	glDeleteBuffers( 1, m_paVBO );
	delete [] m_paVAO; m_paVAO = NULL;
	delete [] m_paVBO; m_paVBO = NULL;
}

//---------------------------------------------------------------
// 
//---------------------------------------------------------------
int Font::OnInit()
{
	m_pShader = ShaderManager::Get()->Font();
	
	Texture::HeaderTbl ht = { 1, &font_tex_header };
	m_hTexture = TextureManager::Get().GetHandle( ht );

	m_paVAO = NEW GLuint[1];
	m_paVBO = NEW GLuint[1];
	
	glGenVertexArrays( 1, m_paVAO );
	glGenBuffers( 1, m_paVBO );

	glBindVertexArray( m_paVAO[0] );
	glBindBuffer( GL_ARRAY_BUFFER, m_paVBO[0] );
	glBufferData( GL_ARRAY_BUFFER,
				  sizeof(GLfloat)*FONT_VERTICES_SIZE,
				  m_Vertices,
				  GL_STATIC_DRAW );
	
	glVertexAttribPointer( m_pShader->LocPosition(), 3, GL_FLOAT, false,
						   sizeof(GL_FLOAT)*FONT_VERTICES_SIZE/4,
						   0 );
	glEnableVertexAttribArray( m_pShader->LocPosition() );


	glVertexAttribPointer( m_pShader->LocUV0(), 2, GL_FLOAT, false,
						   sizeof(GL_FLOAT)*FONT_VERTICES_SIZE/4,
						   reinterpret_cast<void*>(sizeof(float)*3) );
	glEnableVertexAttribArray( m_pShader->LocUV0() );
	
	glBindBuffer( GL_ARRAY_BUFFER, 0 );
	glBindVertexArray( 0 );
	
	return TR_OK;
}

//---------------------------------------------------------------
// Update
//---------------------------------------------------------------
int Font::OnUpdate()
{
	Vector parent_pos = g_ParentPos;

	if( ! m_bConnectParent ) {
		g_ParentPos.x = 0.0f;
		g_ParentPos.y = 0.0f;
	}

	m_ParentPos = g_ParentPos;

	g_ParentPos.x += m_Pos.x;
	g_ParentPos.y += m_Pos.y;

	OnUpdateChildren();
	
	g_ParentPos = parent_pos;
	
	return TR_OK;
}

//---------------------------------------------------------------
// _O
//---------------------------------------------------------------
int Font::OnRender()
{
	if( m_pszStr == NULL ) return TR_OK;

	Vector pos = m_ParentPos + m_Pos;
	float tab;
	
	// tHgeNX`FILL
	m_pShader->Fill();
	Texture * p = TextureManager::Get().GetTexturePtr(m_hTexture);
	if( p ) p->Fill( m_pShader->LocDiffuseMap() );

	Scene::SetModelMatrixPtr( &m );
	const Matrix * pMVP = Scene::GetCurrentModelViewProjMatrixPtr();
	glUniformMatrix4fv( m_pShader->LocMVPMatrix(), 1, GL_FALSE, (float *)pMVP->rc );
	glUniform4fv( m_pShader->LocColor(), 1,  (float *)&(m_Color.r) );
	

	// _obt@BIND
	glBindVertexArray( m_paVAO[0] );
	glBindBuffer( GL_ARRAY_BUFFER, m_paVBO[0] );

	// OpenGLR}h
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	// `	
	for( const char * str = m_pszStr; *str != '\0'; str++ ) {
		if( '\0' <= *str && *str <= ' ' ) {
			//-----------------------
			// R[h
			//-----------------------
			switch( *str ) {
			  case '\n':
				pos.x  = m_ParentPos.x + m_Pos.x;
				pos.y += m_fHeight;
				break;
			  case '\t':
				tab = m_fWidth * (float)FONT_TAB_SIZE;
				pos.x = tab * ((int)(pos.x + tab + 0.5f) / (int)(tab + 0.5f));
				break;
			  case ' ':
				pos.x += m_fWidth;
				break;
			}
		} else {
			//-----------------------
			// R[h
			//-----------------------
			RenderOneLetter( pos, m_Color, *str );
			pos.x += (m_fWidth-1);
		}
	}
	glBindBuffer( GL_ARRAY_BUFFER, 0 );
	glBindVertexArray(0);
	return TR_OK;
}

//---------------------------------------------------------------
// P_O
//---------------------------------------------------------------
void Font::RenderOneLetter( const Vector & p, const Color & c, char l )
{
	float u1 = (float)( ((int)(l-' '-1) & 0x0f) * FONT_TEX_LETTER_X );
	float v1 = (float)( ((int)(l-' '-1) & 0xf0) >> (int)4 ) * FONT_TEX_LETTER_Y;

	float u2 = u1 + FONT_TEX_LETTER_X;
	float v2 = v1 + FONT_TEX_LETTER_Y;

	u1 = (u1+0.5f) / FONT_TEX_WIDTH;
	u2 = (u2+0.5f) / FONT_TEX_WIDTH;
	v1 = (v1+0.5f) / FONT_TEX_HEIGHT;
	v2 = (v2+0.5f) / FONT_TEX_HEIGHT;

	//u1 = 1.0 - u1;
	//u2 = 1.0 - u2;
	v2 = 1.0f - v2;
	v1 = 1.0f - v1;

	float x1 = p.x + 0.5f;
	float y1 = -p.y - 0.5f;

	float x2 = x1 + m_fWidth  + 0.5f;
	float y2 = y1 - m_fHeight - 0.5f;
	float z  = p.z;

	const Matrix * pMVP = Scene::GetCurrentModelViewProjMatrixPtr();

	m_Vertices[0].x  = x1;
	m_Vertices[0].y  = y1;
	m_Vertices[0].z  = z;
	m_Vertices[0].u  = u1;
	m_Vertices[0].v  = v1;

	m_Vertices[1].x  = x2;
	m_Vertices[1].y  = y1;
	m_Vertices[1].z  = z;
	m_Vertices[1].u  = u2;
	m_Vertices[1].v  = v1;

	m_Vertices[2].x  = x2;
	m_Vertices[2].y  = y2;
	m_Vertices[2].z  = z;
	m_Vertices[2].u  = u2;
	m_Vertices[2].v  = v2;

	m_Vertices[3].x  = x1;
	m_Vertices[3].y  = y2;
	m_Vertices[3].z  = z;
	m_Vertices[3].u  = u1;
	m_Vertices[3].v  = v2;
			
	glBufferData( GL_ARRAY_BUFFER,
				  sizeof(GLfloat)*FONT_VERTICES_SIZE,
				  m_Vertices,
				  GL_STATIC_DRAW );


	glEnableVertexAttribArray( m_pShader->LocPosition() );
	glEnableVertexAttribArray( m_pShader->LocUV0()      );
	glDrawArrays( GL_QUADS, 0, 4 );
}

//---------------------------------------------------------------
// Print
//---------------------------------------------------------------
void Font::Print( float x, float y, const char * str )
{
	m_Pos.x = x;
	m_Pos.y = y;
	m_pszStr = str;
}

}
