/** @file font.h
 *  @brief FontNXwb_
 *
 *  @author yasufuku
 *  @date 2004/12
 *  @version 0.10
 *
 *  Copyright(C) 2004 Kensuke Yasufuku
 */
#ifndef _FONT_H_
#define _FONT_H_

#include "include.h"
#include "element.h"
#include "color.h"
#include "vector.h"
#include "texture_manager.h"
#include "shader_base.h"

namespace pge {
	
static const uint FONT_TEX_WIDTH    = 192;
static const uint FONT_TEX_HEIGHT   = 192;
static const uint FONT_TEX_LETTER_X = 12;
static const uint FONT_TEX_LETTER_Y = 24;
static const uint FONT_TAB_SIZE     = 4;
static const uint FONT_VERTICES_SIZE = 20;

//------------------------
//  !"#$%&'()*+,-./0
//  123456789:;<=>?@
//  ABCDEFGHIJKLMNOP
//  QRSTUVWXYZ[\]^_`
//  abcdefghijklmnop
//  qrstuvwxyz{|}~
//------------------------

class Font : public Element
{
private:
	typedef struct _FontVertex {
		float x;
		float y;
		float z;
		float u;
		float v;
	} FontVertex;
	
	const char * m_pszStr;
	bool		 m_bConnectParent;
	bool		 m_bShadow;
	float		 m_fWidth;
	float		 m_fHeight;
	Color		 m_Color;
	Vector		 m_ParentPos;
	Vector		 m_Pos;

	// `֘A
	FontVertex   m_Vertices[4];
	ShaderBase * m_pShader;
	GLuint     * m_paVAO;
	GLuint     * m_paVBO;
	HTexture     m_hTexture;

protected:
	virtual int OnInit();
	virtual int OnUpdate();
	virtual int OnRender();
	void RenderOneLetter( const Vector & p, const Color & c, char l );

public:
	Font();
	virtual ~Font();

	static Font * Create( Element * pParent );
	void   Print( float x, float y, const char * str );
	void   SetSize( float x, float y ){ m_fWidth = x; m_fHeight = y; }
	void   SetColor( const Color & c ){ m_Color = c; }
	void   SetAlpha( float a ){ m_Color.a = a; }
	void   ConnectParent( bool bConnect ){ m_bConnectParent = bConnect; }
	void   SetShadow( bool bShadow ){ m_bShadow = bShadow; }
};

};
#endif //_FONT_H
