/** @file motion.cpp
 *  @brief MotionNX\[X
 *
 *  @author yasufuku
 *  @date 2011/02
 *  @version 0.10
 *
 *  Copyright(C) 2004-2011 Kensuke Yasufuku
 */
#include "motion.h"
#include "mathlib.h"

//====================================================================
// MotionNXo֐`
//====================================================================
//---------------------------------------------------------------
// RXgN^
//---------------------------------------------------------------
/**
 *
 */
Motion::Motion()
{
	m_fFrame = 0.0f;
	m_fPrevFrame = 0.0f;
	m_unEndFrame = 0;
	m_unPrevEndFrame = 0;
	m_pData = NULL;
	m_pPrev = NULL;
	m_pNext = NULL;
	m_pMatrixList = NULL;
	m_fBlendTimer = 0.0f;
	m_fBlendTime  = 0.0f;
	m_fNextBlendTime = 0.0f;
}

//---------------------------------------------------------------
// fXgN^
//---------------------------------------------------------------
/**
 *
 */
Motion::~Motion()
{
	if( m_pMatrixList ) {
		delete [] m_pMatrixList;
	}
}

//---------------------------------------------------------------
// [V^C}[i߂
//---------------------------------------------------------------
void Motion::IncTime( float t )
{
	if( m_unEndFrame > 0 ) {
		m_fFrame += t;

		while( m_fFrame > (float)(m_unEndFrame-1) ) {
			m_fFrame -= (float)(m_unEndFrame);
		}
		while( m_fFrame < 0 ) {
			m_fFrame += (float)(m_unEndFrame);
		}
	}

	if( m_unPrevEndFrame > 0 ) {
		m_fPrevFrame += t;
		
		while( m_fPrevFrame > (float)(m_unPrevEndFrame-1) ) {	
			m_fPrevFrame -= (float)m_unPrevEndFrame;
		}	
		while( m_fPrevFrame < 0 ) {
			m_fPrevFrame += (float)m_unPrevEndFrame;
		}
	}

}

//---------------------------------------------------------------
// f[^Zbg
//---------------------------------------------------------------
void Motion::SetData( MotionData * pData, float fBlend )
{
	if( m_fBlendTimer > 0.000001f ) {
		// uh
		m_pNext = pData;
		m_fNextBlendTime = fBlend;
		return;
	}

	m_fPrevFrame = m_fFrame;
	m_fFrame = 0;

	m_pPrev = m_pData;
	m_unPrevEndFrame = m_unEndFrame;
	
	m_pData = pData;
	m_unEndFrame = pData->unFrame;

	m_fBlendTime  = fBlend;
	m_fBlendTimer = fBlend;
}


s8 * Motion::GetWeightMatrix( s8 * wp, int n )
{
	Mtx44 * pMtx = &m_pMatrixList[n];
	float endcode = 1.0f;
	int blend;
	float ratio;
	float total = 100.0f;


	blend  = *wp++;
	ratio = *wp++;

	endcode = ratio;
	if( ratio < 0.0f ) ratio *= -1.0f;
	total -= ratio;
	
	//*pMtx = m_pMatrixList[blend] * (ratio*0.01f);
	MtxMulScalar( pMtx->m, m_pMatrixList[blend].m, (ratio*0.01f) );
	
	// Qڈȍ~
	while( endcode > 0 )
	{
		blend = *wp ++;
		ratio = *wp ++;
		endcode = ratio;
		//ratio = Fabsf( ratio );
		if( ratio < 0.0f ) ratio *= -1.0f;
		total -= ratio;
		//*pMtx += m_pMatrixList[blend]*(ratio * 0.01f);
		PlusMtxMulScalar( pMtx->m, m_pMatrixList[blend].m, (ratio*0.01f) );
	}
	return wp;
}

//---------------------------------------------------------------
// TRS}gbNXvZ
//---------------------------------------------------------------
void Motion::TrsToMatrix( Mtx44 & m, TRS & trs )
{
	//m.SetTrans( trs.trans[0], trs.trans[1], trs.trans[2] );
	m.m[3][0] = trs.trans[0];
	m.m[3][1] = trs.trans[1];
	m.m[3][2] = trs.trans[2];

	
	float x = trs.quat[0];
	float y = trs.quat[1];
	float z = trs.quat[2];
	float w = trs.quat[3];


	m.m[0][0] = 1.0f-2.0f*(y*y+z*z);
	m.m[0][1] = 2.0f*(x*y-w*z);
	m.m[0][2] = 2.0f*(x*z+w*y);
	m.m[0][3] = 0.0f;
	
	m.m[1][0] = 2.0f*(x*y+w*z);
	m.m[1][1] = 1.0f-2.0f*(x*x+z*z);
	m.m[1][2] = 2.0f*(y*z-w*x);
	m.m[1][3] = 0.0f;
	
	m.m[2][0] = 2.0f*(x*z-w*y);
	m.m[2][1] = 2.0f*(y*z+w*x);
	m.m[2][2] = 1.0f-2.0f*(x*x+y*y);
	m.m[2][3] = 0.0f;

	m.m[3][3] = 1.0f;

	//m.ScaleR( trs.scale[0], trs.scale[1], trs.scale[2] );
	m.m[0][0] *= trs.scale[0];
	m.m[0][1] *= trs.scale[0];
	m.m[0][2] *= trs.scale[0];

	m.m[1][0] *= trs.scale[1];
	m.m[1][1] *= trs.scale[1];
	m.m[1][2] *= trs.scale[1];

	m.m[2][0] *= trs.scale[2];
	m.m[2][1] *= trs.scale[2];
	m.m[2][2] *= trs.scale[2];

	
}

//---------------------------------------------------------------
// TRS̐`
//---------------------------------------------------------------
void Motion::InterpolateTRS( TRS * pBlend,
							 TRS * pKeyA,
							 TRS * pKeyB, float fBlend )
{
	int nBone = (int)m_pData->unBone;
	
	if( fBlend < 0.0001f || fBlend > 0.9999f ) {

		TRS * pCopy = ( fBlend > 0.9999f ) ? pKeyB : pKeyA;
		if( pBlend == pCopy ) return;

		CopyTRS(  nBone, pBlend, pCopy );

		
	} else {

		SlerpQuat( nBone, pBlend, pKeyA, pKeyB, fBlend );
		LerpTrans( nBone, pBlend, pKeyA, pKeyB, fBlend );
		LerpScale( nBone, pBlend, pKeyA, pKeyB, fBlend );
	}
}

//---------------------------------------------------------------
// }gbNXXgXV
//---------------------------------------------------------------
void Motion::UpdateMatrixList( ModelData * pModelData )
{
	int i;
	static TRS trs[100];

	int nMatrixList = pModelData->sBone+pModelData->sWeights;
	
	if( m_pMatrixList == NULL ) {
		m_pMatrixList = NEW Mtx44[nMatrixList];
	}

	// TRS
	{
		if( m_fBlendTimer > 0.000001 ) {

			static TRS pre[100];
			static TRS cur[100];

			{
				float a = (float)((int)m_fFrame);

				if( m_fFrame - a < 0.000001f ) {
					int offset = pModelData->sBone * (int)a;
					CopyTRS(  (int)m_pData->unBone, cur, &m_pData->pTRS[offset] );
				} else {
		
					float b = (float)((int)(m_fFrame+1));
					float t = ( m_fFrame - a )/( b - a );

					if( b > (float)(m_unEndFrame - 1) ) {
						b -= (float)(m_unEndFrame - 1);
					}
			
					int offset1 = pModelData->sBone * (int)a;
					int offset2 = pModelData->sBone * (int)b;

					InterpolateTRS( cur, &m_pData->pTRS[offset1], &m_pData->pTRS[offset2], t );
				}
			}
			{
				float a = (float)((int)m_fPrevFrame);

				if( m_fPrevFrame - a < 0.000001f ) {
					int offset = pModelData->sBone * (int)a;
					CopyTRS(  (int)m_pPrev->unBone, pre, &m_pPrev->pTRS[offset] );
				} else {
		
					float b = (float)((int)(m_fPrevFrame+1));
					float t = ( m_fPrevFrame - a )/( b - a );

					if( b > (float)(m_unPrevEndFrame - 1) ) {
						b -= (float)(m_unPrevEndFrame - 1);
					}
			
					int offset1 = pModelData->sBone * (int)a;
					int offset2 = pModelData->sBone * (int)b;

					InterpolateTRS( pre, &m_pPrev->pTRS[offset1], &m_pPrev->pTRS[offset2], t );
				}
			}

			InterpolateTRS( trs, cur, pre, m_fBlendTimer/m_fBlendTime );

			
			m_fBlendTimer -= 1.0f;

			if( m_pNext && m_fBlendTimer <= 0.000001 ) {
				SetData( m_pNext, m_fNextBlendTime );
				m_pNext = NULL;
			}
			
		} else {

			float a = (float)((int)m_fFrame);

			if( m_fFrame - a < 0.000001f ) {
				int offset = pModelData->sBone * (int)a;
				CopyTRS(  (int)m_pData->unBone, trs, &m_pData->pTRS[offset] );
			} else {
		
				float b = (float)((int)(m_fFrame+1));
				float t = ( m_fFrame - a )/( b - a );

				if( b > (float)(m_unEndFrame - 1) ) {
					b -= (float)(m_unEndFrame - 1);
				}
			
				int offset1 = pModelData->sBone * (int)a;
				int offset2 = pModelData->sBone * (int)b;

				InterpolateTRS( trs, &m_pData->pTRS[offset1], &m_pData->pTRS[offset2], t );
			}
		}

	}
	
	// {[}gbNXXg
	Mtx44 bm;
	
	for( i=0; i<pModelData->sBone; i++ ) {
		TrsToMatrix( bm, trs[i] );
		MtxMulMtx( m_pMatrixList[i].m, bm.m, pModelData->pBoneInvMtx[i].m ); 
	}
}

void Motion::UpdateWeightMatrixList( ModelData * pModelData )
{
	int i;
	// EFCg}gbNX
	s8 * wp   = pModelData->pWeights;
	int mtxNo = pModelData->sBone;
	
	for( i=0; i<pModelData->sWeights; i++ ) {
		wp = GetWeightMatrix( wp, mtxNo++ );

	}
}

//---------------------------------------------------------------
// _obt@ό`
//---------------------------------------------------------------
void Motion::DeformVB( ModelData * pModelData, float ** pVtxBuffer )
{
#if 0
	int i,j;

	MeshData * pMesh  = pModelData->pMesh;
	s16           sCluster = pMesh->sCluster;
	ClusterData * pCluster = pMesh->pCluster;

	
	for( i=0; i<sCluster; i++ ) {

		float * vb1 = pVtxBuffer[pCluster[i].sVtxArrayIndex];
		float * vb2 = pCluster[i].pVtxBuffer;

		for( j=0; j<pCluster[i].nVtxBuffer; j++ ) {			

			int mtx_index = (int)vb2[j*9+8];
			
			if( mtx_index >= 0 ) {

				MulMtxTRS( &vb1[j*9+5],
						   m_pMatrixList[mtx_index].rc,
						   &vb2[j*9+5] );
				
				MulMtxRS ( &vb1[j*9+2],
						   m_pMatrixList[mtx_index].rc,
						   &vb2[j*9+2] );

			} else {
				
				vb1[j*8+5] = vb2[j*8+5];
				vb1[j*8+6] = vb2[j*8+6];
				vb1[j*8+7] = vb2[j*8+7];

				vb1[j*8+2] = vb2[j*8+2];
				vb1[j*8+3] = vb2[j*8+3];
				vb1[j*8+4] = vb2[j*8+4];
			}
		}
	}
#endif
}
