/** @file motion.h
 *  @brief MotionNXwb_
 *
 *  @author yasufuku
 *  @date 2011/02
 *  @version 0.10
 *
 *  Copyright(C) 2004-2011 Kensuke Yasufuku
 */
#ifndef _MOTION_H_
#define _MOTION_H_

#include "include.h"
#include "model_data.h"
#include "matrix.h"

/**
 *  @brief MotionNX
 *
 *	[VNX
 */
class Motion
{
private:
	float m_fFrame;
	float m_fPrevFrame;
	int   m_unEndFrame;
	int   m_unPrevEndFrame;
	MotionData * m_pData;		///< [Vf[^
	MotionData * m_pPrev;
	Mtx44      * m_pMatrixList;

	float m_fBlendTimer;
	float m_fBlendTime;			///< uh(frameP)

	MotionData * m_pNext;
	float        m_fNextBlendTime;

protected:
	void InterpolateTRS( TRS * pBlend, TRS * pKeyA, TRS * pKeyB, float fBlend );
	s8 * GetWeightMatrix( s8 * wp, int n );

public:
	Motion();					///< RXgN^
	virtual ~Motion();			///< fXgN^

	void IncTime( float t = 1.0f );
	void SetData( MotionData * pData, float fBlend );
	void TrsToMatrix( Mtx44 & m, TRS & trs );
	void UpdateMatrixList( ModelData * pModelData );
	void UpdateWeightMatrixList( ModelData * pModelData );
	void DeformVB( ModelData * pModelData, float ** pVtxBuffer );
	Mtx44 * GetMatrixList(){ return m_pMatrixList; }
};

#endif //_MOTION_H_
