/*  @brief PrimitiveNXCvg
 *
 *  @author yasufuku
 *  @date 2011/02
 *  @version 0.20
 *
 *  Copyright(C) 2007 Kensuke Yasufuku
 */
#include "primitive.h"
#include "shader_manager.h"
#include "scene.h"

extern ModelData cone_model;
extern ModelData cylinder_model;

extern ModelData arrow_line_model;
extern ModelData arrow_end_model;

namespace pge {
//===============================================================
// LineNXo֐
//===============================================================
//---------------------------------------------------------------
// 
//---------------------------------------------------------------
Line * Line::Create( Element * pParent,
					 const Vector & s,
					 const Vector & e,
					 const Color  & c )
{
	Line * pLine = NEW Line();
	pLine->Init( pParent, TP_MODEL );
	pLine->SetColor( c );
	pLine->SetPoints( s, e );
	return pLine;
}

//---------------------------------------------------------------
// RXgN^
//---------------------------------------------------------------
Line::Line()
{
	int i;
	SetName("Line");
	m_pShader = NULL;
	for( i=0; i<6; i++ ) {
		m_Vertices[i] = 0.0f;
	}
}

//---------------------------------------------------------------
// fXgN^
//---------------------------------------------------------------
Line::~Line()
{
	glDeleteVertexArrays( 1, m_paVAO );
	glDeleteBuffers( 1, m_paVBO );
	delete [] m_paVAO; m_paVAO = NULL;
	delete [] m_paVBO; m_paVBO = NULL;
}

//---------------------------------------------------------------
// 
//---------------------------------------------------------------
int Line::OnInit()
{
	m_pShader = ShaderManager::Get()->StaticColor();

	m_paVAO = NEW GLuint[1];
	m_paVBO = NEW GLuint[1];
	
	glGenVertexArrays( 1, m_paVAO );
	glGenBuffers( 1, m_paVBO );

	glBindVertexArray( m_paVAO[0] );
	glBindBuffer( GL_ARRAY_BUFFER, m_paVBO[0] );
	glBufferData( GL_ARRAY_BUFFER,
				  sizeof(GLfloat)*2*3,
				  m_Vertices,
				  GL_STATIC_DRAW );
	
	glVertexAttribPointer( m_pShader->LocPosition(), 3, GL_FLOAT, false, sizeof(float)*3, 0 );
	glEnableVertexAttribArray( m_pShader->LocPosition() );
	
	
	glBindBuffer( GL_ARRAY_BUFFER, 0 );
	glBindVertexArray( 0 );
		
	return TR_OK;
}

//---------------------------------------------------------------
// _O
//---------------------------------------------------------------
int Line::OnRender()
{
	Scene::SetModelMatrixPtr( &m );
	const Matrix * pMVP = Scene::GetCurrentModelViewProjMatrixPtr();

	m_pShader->Fill();
	glBindVertexArray( m_paVAO[0] );
	glBindBuffer( GL_ARRAY_BUFFER, m_paVBO[0] );	
	glEnableVertexAttribArray( m_pShader->LocPosition() );
	glUniformMatrix4fv( m_pShader->LocMVPMatrix(), 1, GL_FALSE, (float *)pMVP->rc );
	glUniform4fv( m_pShader->LocColor(), 1, (float *)&m_Color.r );
	glDrawArrays( GL_LINES, 0, 2 );
	
	glBindBuffer( GL_ARRAY_BUFFER, 0 );
	glBindVertexArray(0);

	return TR_OK;
};

//---------------------------------------------------------------
// n_AI_̎w
//---------------------------------------------------------------
void Line::SetPoints( const Vector & s, const Vector & e )
{
	m_Vertices[0] = s.x;
	m_Vertices[1] = s.y;
	m_Vertices[2] = s.z;
	m_Vertices[3] = e.x;
	m_Vertices[4] = e.y;
	m_Vertices[5] = e.z;
	
	glBindBuffer( GL_ARRAY_BUFFER, m_paVBO[0] );
	glBufferData( GL_ARRAY_BUFFER,
				  sizeof(GLfloat)*2*4,
				  m_Vertices,
				  GL_STATIC_DRAW );
}
	
//---------------------------------------------------------------
// J[ݒ
//---------------------------------------------------------------
void Line::SetColor( const Color & c )
{
	m_Color = c;
}


//===============================================================
// ArrowNXo֐
//===============================================================
//---------------------------------------------------------------
// 
//---------------------------------------------------------------
Arrow * Arrow::Create( Element * pParent,
					 const Vector & s,
					 const Vector & e,
					 const Color  & c )
{
	Arrow * pArrow = NEW Arrow();
	pArrow->Init( pParent, TP_MODEL );
	pArrow->SetColor( c );
	pArrow->SetPoints( s, e );
	return pArrow;
}

//---------------------------------------------------------------
// RXgN^
//---------------------------------------------------------------
Arrow::Arrow()
{
	SetName("Arrow");
	m_fLength = 0.0f;
	m_paEBO = NULL;
}

//---------------------------------------------------------------
// fXgN^
//---------------------------------------------------------------
Arrow::~Arrow()
{
	glDeleteVertexArrays( 2, m_paVAO );
	glDeleteBuffers( 2, m_paVBO );
	glDeleteBuffers( 2, m_paEBO );
	delete [] m_paVAO; m_paVAO = NULL;
	delete [] m_paVBO; m_paVBO = NULL;
	delete [] m_paEBO; m_paEBO = NULL;
}

//---------------------------------------------------------------
// 
//---------------------------------------------------------------
int Arrow::OnInit()
{
	m_pShader = ShaderManager::Get()->StaticColor();

	ClusterData * pData[2];
	pData[0] = arrow_end_model.pMesh->pCluster;
	pData[1] = arrow_line_model.pMesh->pCluster;
	
	m_paVAO = NEW GLuint[2];
	m_paVBO = NEW GLuint[2];
	m_paEBO = NEW GLuint[2];
	
	int VSize = 9;

	glGenVertexArrays( 2, m_paVAO );
	glGenBuffers( 2, m_paVBO );
	glGenBuffers( 2, m_paEBO );

	for( int i=0; i<2; i++ ) {
		glBindVertexArray( m_paVAO[i] );
		glBindBuffer( GL_ARRAY_BUFFER, m_paVBO[i] );
		glBufferData( GL_ARRAY_BUFFER,
					  pData[i]->nVtxBuffer*sizeof(float)*VSize,
					  pData[i]->pVtxBuffer,
					  GL_STATIC_DRAW );
	
		glVertexAttribPointer( m_pShader->LocPosition(), 3, GL_FLOAT, false, sizeof(float)*VSize, reinterpret_cast<void*>(sizeof(float)*5) );
		glEnableVertexAttribArray( m_pShader->LocPosition() );
	
		// CfbNXobt@쐬
		glBindBuffer( GL_ELEMENT_ARRAY_BUFFER, m_paEBO[i] );
		glBufferData( GL_ELEMENT_ARRAY_BUFFER,
					  pData[i]->nIdxBuffer*sizeof(int),
					  pData[i]->pIdxBuffer,
					  GL_STATIC_DRAW );	
	}
	
	glBindVertexArray( 0 );
	glBindBuffer( GL_ELEMENT_ARRAY_BUFFER, 0 );
	glBindBuffer( GL_ARRAY_BUFFER, 0 );

	
	return TR_OK;
}

//---------------------------------------------------------------
// _O
//---------------------------------------------------------------
int Arrow::OnRender()
{
	Matrix tm;

	ClusterData * pData[2];
	pData[0] = arrow_end_model.pMesh->pCluster;
	pData[1] = arrow_line_model.pMesh->pCluster;

	m_pShader->Fill();
	glEnableVertexAttribArray( m_pShader->LocPosition() );
	glUniform4fv( m_pShader->LocColor(), 1, (float *)&m_Color.r );

	// ̐[
	{
		tm.Unit();
		tm.TransR( 0.0f, 0.0f, -(m_fLength-1.0f) );
		tm = m_mLookat * tm;

	
		glBindVertexArray( m_paVAO[0] );
		glBindBuffer( GL_ARRAY_BUFFER, m_paVBO[0] );	
		glBindBuffer( GL_ELEMENT_ARRAY_BUFFER, m_paEBO[0] );
	
		Scene::SetModelMatrixPtr( &tm );
		const Matrix * pMVP = Scene::GetCurrentModelViewProjMatrixPtr();	
		glUniformMatrix4fv( m_pShader->LocMVPMatrix(), 1, GL_FALSE, (float *)pMVP->rc );
		
	
		glDrawElements( GL_TRIANGLES, pData[0]->nIdxBuffer, GL_UNSIGNED_INT, 0 );
	}
	// ̖_
	{
		tm.Unit();
		tm.ScaleR( 1.0f, 1.0f, m_fLength/0.85f-0.15f );
		tm = m_mLookat * tm;


	
		glBindVertexArray( m_paVAO[1] );
		glBindBuffer( GL_ARRAY_BUFFER, m_paVBO[1] );	
		glBindBuffer( GL_ELEMENT_ARRAY_BUFFER, m_paEBO[1] );
	
		Scene::SetModelMatrixPtr( &tm );
		const Matrix * pMVP = Scene::GetCurrentModelViewProjMatrixPtr();	
		glUniformMatrix4fv( m_pShader->LocMVPMatrix(), 1, GL_FALSE, (float *)pMVP->rc );
		
	
		glDrawElements( GL_TRIANGLES, pData[1]->nIdxBuffer, GL_UNSIGNED_INT, 0 );
	}

	
	glBindVertexArray(0);
	glBindBuffer( GL_ARRAY_BUFFER, 0 );
	glBindBuffer( GL_ELEMENT_ARRAY_BUFFER, 0 );

	return TR_OK;
};

//---------------------------------------------------------------
// n_AI_̎w
//---------------------------------------------------------------
void Arrow::SetPoints( const Vector & s, const Vector & e )
{
	m_vStart = s;
	m_fLength = (e-s).Length();
	m_mLookat.Lookat( s, e, Vector( 0.0f, 1.0f, 0.0f ) );
	m_mLookat.Invert();
}
	
//---------------------------------------------------------------
// J[ݒ
//---------------------------------------------------------------
void Arrow::SetColor( const Color & c )
{
	m_Color = c;
}

};
