/** @file scene.h
 *  @brief SceneNXwb_
 *
 *  @author yasufuku
 *  @date 2011/02
 *  @version 0.10
 *
 *  Copyright(C) 2004-2011 Kensuke Yasufuku
 */
#ifndef _SCENE_H_
#define _SCENE_H_

#include "element.h"
#include "view.h"
#include "matrix.h"
#include "camera.h"
#include "light.h"

/**
 *	  @brief SceneNX
 *
 */
class Scene : public Element
{
private:
	static Scene * s_pCurScene;
	
	const Camera * m_pCamera;
	const Light  * m_pLight;

	const Matrix * m_pProjMatrix;
	const Matrix * m_pViewMatrix;
	const Matrix * m_pModelMatrix;
	Matrix   m_ViewProjMatrix;
	Matrix   m_ModelViewProjMatrix;
	Matrix   m_ModelViewMatrix;


protected:
	Scene();					///< RXgN^
	virtual ~Scene();			///< fXgN^
	
	virtual int OnUpdate();		///< XV
	virtual int OnRender();		///< _O
public:
	/// ֐
	static Scene * Create( Element * pParent, int nPriority = TP_SCENE );
	
	// Setter
	static void SetViewProjMatrixPtr( const Matrix * proj, const Matrix * view  );
	static void SetModelMatrixPtr( const Matrix * model );
	static void SetCameraPtr( const Camera * pCamera );
	static void SetLightPtr( const Light * pLight );

	//Getter
	static Scene * GetCurrent(){ return s_pCurScene; }
	static const Matrix * GetCurrentPtr(){ return &s_pCurScene->m_ViewProjMatrix; }
	static const Matrix * GetCurrentModelViewProjMatrixPtr(){ return &s_pCurScene->m_ModelViewProjMatrix; }
	static const Matrix * GetCurrentModelViewMatrixPtr(){ return &s_pCurScene->m_ModelViewMatrix; }
	static const Matrix * GetCurrentModelMatrixPtr(){ return s_pCurScene->m_pModelMatrix; }
	static const Camera * GetCurrentCameraPtr(){ return s_pCurScene->m_pCamera; }
	static const Light  * GetCurrentLightPtr(){ return s_pCurScene->m_pLight; }
};

#endif //_SCENE_H_
