/** @file shader_base.cpp
 *  @brief ShaderBaseNX\[X
 *
 *  @author yasufuku
 *  @date 2011/01
 *  @version 0.10
 *
 *  Copyright(C) 2011 Kensuke Yasufuku
 */

#include "shader_base.h"

#define SHADER_VNAME_POSITION    "Position"		 // VF[_[̕ϐ
#define SHADER_VNAME_NORMAL      "Normal"		 // VF[_[̕ϐ
#define SHADER_VNAME_INDEX       "Index"		 // VF[_[̕ϐ
#define SHADER_VNAME_UV0         "UV0"			 // VF[_[̕ϐ
#define SHADER_VNAME_UV1         "UV1"			 // VF[_[̕ϐ
#define SHADER_VNAME_VCOLOR      "VColor"		 // VF[_[̕ϐ
#define SHADER_VNAME_MVPMATRIX   "MVPMatrix"	 // VF[_[̕ϐ(MVP=ModelViewProjection)
#define SHADER_VNAME_WORLDMATRIX "WorldMatrix"	 // VF[_[̕ϐ
#define SHADER_VNAME_MVMATRIX	 "MVMatrix"		 // VF[_[̕ϐ(MV =ModelView)
#define SHADER_VNAME_PROJMATRIX  "ProjMatrix"	 // VF[_[̕ϐ
#define SHADER_VNAME_MATRIXLIST  "MatrixList"	 // VF[_[̕ϐ
#define SHADER_VNAME_EYE_POS     "EyePos"		 // VF[_[̕ϐ
#define SHADER_VNAME_WORLDMATRIX "WorldMatrix"	 // VF[_[̕ϐ
#define SHADER_VNAME_DIFFUSE     "Diffuse"		 // VF[_[̕ϐ
#define SHADER_VNAME_AMBIENT     "Ambient"		 // VF[_[̕ϐ
#define SHADER_VNAME_SPECULAR    "Specular"		 // VF[_[̕ϐ
#define SHADER_VNAME_COLOR       "Color"		 // VF[_[̕ϐ
#define SHADER_VNAME_LIGHTS_NUM  "LightNum"		 // VF[_[̕ϐ
#define SHADER_VNAME_LIGHTS_POS  "LightPos"      // VF[_[̕ϐ
#define SHADER_VNAME_LIGHTS_DIR  "LightDir"      // VF[_[̕ϐ
#define SHADER_VNAME_LIGHTS_INT  "LightIntensity" // VF[_[̕ϐ
#define SHADER_VNAME_DIFFUSE_MAP "DiffuseMap"    // VF[_[̕ϐ
#define SHADER_VNAME_LIGHT_MAP   "LightMap"      // VF[_[̕ϐ

//====================================================================
// ֐
//====================================================================
//--------------------------------------------------------------
// VF[_[̃\[XvOɓǂݍ
//--------------------------------------------------------------
int readShaderSource(GLuint shader, const char *file)
{
  FILE *fp;
  const GLchar *source;
  GLsizei length;
  int ret;
  
  /* t@CJ */
  fp = fopen(file, "rb");
  if (fp == NULL) {
    perror(file);
    return -1;
  }
  
  /* t@C̖Ɉړ݈ʒui܂t@CTCYj𓾂 */
  fseek(fp, 0L, SEEK_END);
  length = ftell(fp);
  
  /* t@CTCỸm */
  source = (GLchar *)malloc(length);
  if (source == NULL) {
    fprintf(stderr, "Could not allocate read buffer.\n");
    return -1;
  }
  
  /* t@C擪ǂݍ */
  fseek(fp, 0L, SEEK_SET);
  ret = fread((void *)source, 1, length, fp) != (size_t)length;
  fclose(fp);
  
  /* VF[_̃\[XvÕVF[_IuWFNgւ̓ǂݍ */
  if (ret)
    fprintf(stderr, "Could not read file: %s.\n", file);
  else
    glShaderSource(shader, 1, &source, &length);
  
  /* mۂ̊J */
  free((void *)source);
  
  return ret;
}

//--------------------------------------------------------------
// VF[_[\
//--------------------------------------------------------------
void printShaderInfoLog(GLuint shader)
{
  GLsizei bufSize;
  
  glGetShaderiv(shader, GL_INFO_LOG_LENGTH , &bufSize);
  
  if (bufSize > 1) {
    GLchar *infoLog;
    
    infoLog = (GLchar *)malloc(bufSize);
    if (infoLog != NULL) {
      GLsizei length;
      
      glGetShaderInfoLog(shader, bufSize, &length, infoLog);
      fprintf(stderr, "InfoLog:\n%s\n\n", infoLog);
      free(infoLog);
    }
    else
      fprintf(stderr, "Could not allocate InfoLog buffer.\n");
  }
}

//--------------------------------------------------------------
// vȌ\
//--------------------------------------------------------------
void printProgramInfoLog(GLuint program)
{
  GLsizei bufSize;
  
  glGetProgramiv(program, GL_INFO_LOG_LENGTH , &bufSize);
  
  if (bufSize > 1) {
    GLchar *infoLog;
    
    infoLog = (GLchar *)malloc(bufSize);
    if (infoLog != NULL) {
      GLsizei length;
      
      glGetProgramInfoLog(program, bufSize, &length, infoLog);
      fprintf(stderr, "InfoLog:\n%s\n\n", infoLog);
      free(infoLog);
    }
    else
      fprintf(stderr, "Could not allocate InfoLog buffer.\n");
  }
}


//====================================================================
// ShaderBaseNXo֐`
//====================================================================
//--------------------------------------------------------------
// RXgN^
//--------------------------------------------------------------
ShaderBase::ShaderBase(  const char * vsfname, const char * fsname )
:
m_pszVsfname(vsfname),
m_pszFsfname(fsname)
{
	// [NA
	m_hProgram = 0;
	m_hLocPosition    = -1;
	m_hLocNormal      = -1;
	m_hLocIndex       = -1;
	m_hLocUV0         = -1;
	m_hLocUV1         = -1;
	m_hLocVColor      = -1;
	m_hLocMVPMatrix   = -1;
	m_hLocWorldMatrix = -1;
	m_hLocMVMatrix    = -1;
	m_hLocProjMatrix  = -1;
	m_hLocMatrixList  = -1;
	m_hLocEyePos      = -1;

	m_hLocDiffuse  = -1;
	m_hLocAmbient  = -1;
	m_hLocSpecular = -1;
	m_hLocColor    = -1;

	m_hLocLightsNum = -1;
	m_hLocLightsPos = -1;
	m_hLocLightsInt = -1;

	m_hLocDiffuseMap  = -1;
	m_hLocLightMap  = -1;

	Load();
}

//--------------------------------------------------------------
// fXgN^
//--------------------------------------------------------------
ShaderBase::~ShaderBase()
{
	Unload();
}

//--------------------------------------------------------------
// [h
//--------------------------------------------------------------
int ShaderBase::Load()
{
	GLint compiled, linked;
		
	// VF[_[IuWFNg̐
	GLuint VertShader = glCreateShader( GL_VERTEX_SHADER   );
	GLuint FragShader = glCreateShader( GL_FRAGMENT_SHADER );

	// VF[_̃\[XvO̓ǂݍ
	if( readShaderSource( VertShader, m_pszVsfname ) ) return RET_FILE_NOT_FOUND;
	if( readShaderSource( FragShader, m_pszFsfname ) ) return RET_FILE_NOT_FOUND;

	// o[ebNXVF[_̃\[XvÕRpC
	glCompileShader( VertShader );
	glGetShaderiv( VertShader, GL_COMPILE_STATUS, &compiled );
	printShaderInfoLog( VertShader );
	if( compiled == GL_FALSE ) {
		printf("Compile error in vertex shader.\n");
		assert(false);
		return RET_COMPILE_ERROR;
	}

	// tOgVF[_̃\[XvÕRpC
	glCompileShader( FragShader);
	glGetShaderiv( FragShader, GL_COMPILE_STATUS, &compiled );
	printShaderInfoLog( FragShader );
	if( compiled == GL_FALSE ) {
		printf("Compile error in fragment shader.\n");
		assert(false);
		return RET_COMPILE_ERROR;
	}
  
	// vOIuWFNg̍쐬
	m_hProgram = glCreateProgram();
  
	// VF[_IuWFNg̃VF[_vOւ̓o^
	glAttachShader( m_hProgram, VertShader );
	glAttachShader( m_hProgram, FragShader );
  
	// VF[_IuWFNg̍폜
	glDeleteShader( VertShader );
	glDeleteShader( FragShader );
  
	// VF[_vÕN
	glLinkProgram( m_hProgram );
	glGetProgramiv( m_hProgram, GL_LINK_STATUS, &linked );
	printProgramInfoLog( m_hProgram );
	if (linked == GL_FALSE) {
		PRINTF("Link error.\n");
		return RET_LINK_ERROR;
	}

	// VF[_[vOƂ̊֘At
	m_hLocPosition    = glGetAttribLocation( m_hProgram, SHADER_VNAME_POSITION     );
	m_hLocNormal      = glGetAttribLocation( m_hProgram, SHADER_VNAME_NORMAL       );
	m_hLocIndex       = glGetAttribLocation( m_hProgram, SHADER_VNAME_INDEX        );
	m_hLocUV0         = glGetAttribLocation( m_hProgram, SHADER_VNAME_UV0          );
	m_hLocUV1         = glGetAttribLocation( m_hProgram, SHADER_VNAME_UV1          );
	m_hLocVColor      = glGetAttribLocation( m_hProgram, SHADER_VNAME_VCOLOR       );
	m_hLocMVPMatrix   = glGetUniformLocation( m_hProgram, SHADER_VNAME_MVPMATRIX   );
	m_hLocWorldMatrix = glGetUniformLocation( m_hProgram, SHADER_VNAME_WORLDMATRIX );
	m_hLocMVMatrix    = glGetUniformLocation( m_hProgram, SHADER_VNAME_MVMATRIX    );
	m_hLocProjMatrix  = glGetUniformLocation( m_hProgram, SHADER_VNAME_PROJMATRIX  );
	m_hLocMatrixList  = glGetUniformLocation( m_hProgram, SHADER_VNAME_MATRIXLIST  );
	m_hLocEyePos      = glGetUniformLocation( m_hProgram, SHADER_VNAME_EYE_POS     );
	m_hLocDiffuse     = glGetUniformLocation( m_hProgram, SHADER_VNAME_DIFFUSE     );
	m_hLocAmbient     = glGetUniformLocation( m_hProgram, SHADER_VNAME_AMBIENT     );
	m_hLocSpecular    = glGetUniformLocation( m_hProgram, SHADER_VNAME_SPECULAR    );
	m_hLocColor       = glGetUniformLocation( m_hProgram, SHADER_VNAME_COLOR       );
	m_hLocLightsNum   = glGetUniformLocation( m_hProgram, SHADER_VNAME_LIGHTS_NUM  );
	m_hLocLightsPos   = glGetUniformLocation( m_hProgram, SHADER_VNAME_LIGHTS_POS  );
	m_hLocLightsDir   = glGetUniformLocation( m_hProgram, SHADER_VNAME_LIGHTS_DIR  );
	m_hLocLightsInt   = glGetUniformLocation( m_hProgram, SHADER_VNAME_LIGHTS_INT  );
	m_hLocDiffuseMap  = glGetUniformLocation( m_hProgram, SHADER_VNAME_DIFFUSE_MAP );
	m_hLocLightMap    = glGetUniformLocation( m_hProgram, SHADER_VNAME_LIGHT_MAP   );

	return RET_LOAD_OK;
}

//--------------------------------------------------------------
// A[h
//--------------------------------------------------------------
int ShaderBase::Unload()
{
	glDeleteProgram( m_hProgram );
	return 0;
}

//--------------------------------------------------------------
// FILL
//--------------------------------------------------------------
void ShaderBase::Fill()
{
	glUseProgram( m_hProgram );
}

//--------------------------------------------------------------
// \
//--------------------------------------------------------------
void ShaderBase::PrintInfo()
{
	//
}
