/** @file skin_mesh.cpp
 *  @brief SkinMeshNX
 *
 *  @author yasufuku
 *  @date 2011/02
 *  @version 0.20
 *
 *  Copyright(C) 2004-2011 Kensuke Yasufuku
 */
#include "skin_mesh.h"
#include "scene.h"
#include "vector.h"
#include "shader_manager.h"
#include "keyboard.h"

//===============================================================
// SkinMeshNXo֐`
//===============================================================
//---------------------------------------------------------------
// 
//---------------------------------------------------------------
SkinMesh * SkinMesh::Create( MeshData * pData, s16 sBone, s16 sWeight )
{
	SkinMesh * p = NEW SkinMesh;
	p->m_sBone   = sBone;
	p->m_sWeight = sWeight;
	p->Init( pData );
	
	return p;
}

//---------------------------------------------------------------
// RXgN^
//---------------------------------------------------------------
SkinMesh::SkinMesh()
{
	// [NA
	m_pData = NULL;

	m_pVAO = NULL;
	m_pVBO = NULL;
	m_pEBO = NULL;

	m_unVertex  = 0;
	m_unPolygon = 0;
	m_unTex     = 0;
	m_unTexSize = 0;
}

//---------------------------------------------------------------
// fXgN^
//---------------------------------------------------------------
SkinMesh::~SkinMesh()
{
	int i;
	
	// }eA}l[WŊǗ\肾A
	// Ƃ肠bVƂɃ}eA𐶐
	for( i=0; i<m_pData->sMaterial; i++ ) {
		delete m_paMaterialTbl[i];
	}
	delete [] m_paMaterialTbl;

	DeleteVertexBuffer();
}

//---------------------------------------------------------------
// 
//---------------------------------------------------------------
int SkinMesh::Init( MeshData * pData )
{
	m_pData = pData;

	eShaderMeshType stype = SKIN_MESH_SHADER;
	if( m_sBone == 1 ) stype = STATIC_MESH_SHADER; // BoneNull݂̂̏ꍇStaticMeshɂĂ
	
	// }eA}l[WŊǗ\肾A
	// Ƃ肠bVƂɃ}eA𐶐
	m_paMaterialTbl = (Material **)NEW u32[m_pData->sMaterial];
	for( int i=0; i<m_pData->sMaterial; i++ ) {
		m_paMaterialTbl[i] = Material::Create( &m_pData->pMaterial[i], stype );
		m_unTex     += m_paMaterialTbl[i]->GetTexNum();
		m_unTexSize += m_paMaterialTbl[i]->GetTexSize();
	}

	// _zIuWFNgƒ_obt@IuWFNg̍쐬
	MakeVertexBuffer();

	// }eAƂɐݒ肳ĂVF[_[ŏdĂ̂
	// ̂ŏd̂ȂVF[_[Xg쐬Ă
	MakeShaderList();
	
	return 0;
}

//---------------------------------------------------------------
// _obt@쐬
//---------------------------------------------------------------
void SkinMesh::MakeVertexBuffer()
{
	int i;
	ClusterData * pCluster = m_pData->pCluster;
	ShaderBase * pShader;

	// _zIuWFNg(OpenGL 3.0ȍ~)𗘗p
	m_pVAO = NEW GLuint[m_pData->sVArray];
	m_pVBO = NEW GLuint[m_pData->sVArray];
	m_pEBO = NEW GLuint[m_pData->sVArray];

	glGenVertexArrays( m_pData->sVArray, m_pVAO );
	glGenBuffers( m_pData->sVArray, m_pVBO );
	glGenBuffers( m_pData->sVArray, m_pEBO );

	u32 VSize; // 1_̃TCY
	u32 offset;
	
	for( i=0; i<m_pData->sVArray; i++ ) {

		VSize   = GetVertexSize( pCluster[i].unVtxFormat );
		offset  = 0;
		pShader = m_paMaterialTbl[pCluster[i].sMaterialIndex]->GetShader();

		//----------------------------------------------------
		// _obt@쐬
		//----------------------------------------------------
		glBindVertexArray( m_pVAO[i] );
		
		glBindBuffer( GL_ARRAY_BUFFER, m_pVBO[i] );
		glBufferData( GL_ARRAY_BUFFER,
					  pCluster[i].nVtxBuffer*sizeof(float)*VSize,
					  pCluster[i].pVtxBuffer,
					  GL_STATIC_DRAW );

		// _tH[}bgȂVF[_[Ƃ̊֘At
		// tH[}bgƂ肠2ށij
		//  0: ʒuW + sCfbNX + @ + UV0
		//  1: ʒuW + sCfbNX + @ + UV0 + UV1 eNX`2
		if( pCluster[i].unVtxFormat == 0 || pCluster[i].unVtxFormat == 1 ) {
			// 1ڂ̃eNX`UV
			glVertexAttribPointer( pShader->LocUV0(), 2, GL_FLOAT, false, sizeof(float)*VSize, reinterpret_cast<void*>(sizeof(float)*offset) );
			glEnableVertexAttribArray( pShader->LocUV0() );
		}
		offset += 2;
		if( pCluster[i].unVtxFormat == 1 ) {
			// 2ڂ̃eNX`UV
			glVertexAttribPointer( pShader->LocUV1(), 2, GL_FLOAT, false, sizeof(float)*VSize, reinterpret_cast<void*>(sizeof(float)*offset) );
			glEnableVertexAttribArray( pShader->LocUV1() );
			offset += 2;
		}
		// @
		glVertexAttribPointer( pShader->LocNormal(), 3, GL_FLOAT, false, sizeof(float)*VSize, reinterpret_cast<void*>(sizeof(float)*offset) );
		glEnableVertexAttribArray( pShader->LocNormal() );
		offset += 3;
		// ʒuW
		glVertexAttribPointer( pShader->LocPosition(), 3, GL_FLOAT, false, sizeof(float)*VSize, reinterpret_cast<void*>(sizeof(float)*offset) );
		glEnableVertexAttribArray( pShader->LocPosition() );
		offset += 3;
		// sCfbNX x1
		if( pShader->LocIndex() >= 0 ) {
			glVertexAttribPointer( pShader->LocIndex(), 1, GL_FLOAT, false, sizeof(float)*VSize, reinterpret_cast<void*>(sizeof(float)*offset) );
			glEnableVertexAttribArray( pShader->LocIndex() );
		}		

		//----------------------------------------------------
		// CfbNXobt@쐬
		//----------------------------------------------------
		glBindBuffer( GL_ELEMENT_ARRAY_BUFFER, m_pEBO[i] );
		glBufferData( GL_ELEMENT_ARRAY_BUFFER,
					  pCluster[i].nIdxBuffer*sizeof(int),
					  pCluster[i].pIdxBuffer,
					  GL_STATIC_DRAW );
		
		
		// _JEg
		m_unVertex  += pCluster[i].nVtxBuffer;
		// |SJEg
		if( pCluster[i].nPrimType == MVF_TRIANGLE_STRIP ) {
			m_unPolygon += pCluster[i].nIdxBuffer + 2;
		} else {
			m_unPolygon += pCluster[i].nIdxBuffer/3;
		}
		
	}
	glBindBuffer( GL_ARRAY_BUFFER, 0 );
	glBindBuffer( GL_ELEMENT_ARRAY_BUFFER, 0 );
	glBindVertexArray( 0 );
}

//---------------------------------------------------------------
// _obt@
//---------------------------------------------------------------
void SkinMesh::DeleteVertexBuffer()
{
	for( int i=0; i<m_pData->sVArray; i++ ) {
		glDeleteVertexArrays( m_pData->sVArray, m_pVAO );
		glDeleteBuffers( m_pData->sVArray, m_pVBO );
		glDeleteBuffers( m_pData->sVArray, m_pEBO );
	}
	
	delete [] m_pVAO; m_pVAO = NULL;
	delete [] m_pVBO; m_pVBO = NULL;
	delete [] m_pEBO; m_pEBO = NULL;
}

//---------------------------------------------------------------
// [Vf[^]
//---------------------------------------------------------------
void SkinMesh::FillMotion( Mtx44 * pMatrixList, int num )
{
	for( u32 i=0; i<m_unShader; i++ ) {
		m_paShaderList[i]->Fill();
		glUniformMatrix4fv( glGetUniformLocation( m_paShaderList[i]->Program(), "MatrixList"  ), num, GL_FALSE, (float *)pMatrixList->m );
	}
}

//---------------------------------------------------------------
// `
//---------------------------------------------------------------
void SkinMesh::Fill()
{
	u32 i;

	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK); 

	const Matrix * pMVP  = Scene::GetCurrentModelViewProjMatrixPtr();
	const Matrix * pMdl  = Scene::GetCurrentModelMatrixPtr();
	const Matrix * pMV   = Scene::GetCurrentModelViewMatrixPtr();
	const Matrix * pProj = &Scene::GetCurrentCameraPtr()->proj;
	const Vector * pCam  = &Scene::GetCurrentCameraPtr()->t;
	Light  * pLight = (Light *)Scene::GetCurrentLightPtr();
	
	// ModelViewProjection}gbNXZbg
	for( i=0; i<m_unShader; i++ ) {
		m_paShaderList[i]->Fill();

		// VF[_[Ƃ̊֘At
		if( m_paShaderList[i]->LocUV0() > 0 ) {
			glEnableVertexAttribArray( m_paShaderList[i]->LocUV0() );
		}
		// VF[_[Ƃ̊֘At
		if( m_paShaderList[i]->LocUV1() > 0 ) {
			glEnableVertexAttribArray( m_paShaderList[i]->LocUV1() );
		}
		glEnableVertexAttribArray( m_paShaderList[i]->LocNormal() );
		glEnableVertexAttribArray( m_paShaderList[i]->LocPosition() );
				
		glUniformMatrix4fv( m_paShaderList[i]->LocMVPMatrix(),   1, GL_FALSE, (float *)pMVP->rc );
		glUniformMatrix4fv( m_paShaderList[i]->LocWorldMatrix(), 1, GL_FALSE, (float *)pMdl->rc );
		glUniformMatrix4fv( m_paShaderList[i]->LocMVMatrix(),    1, GL_FALSE, (float *)pMV->rc );
		glUniformMatrix4fv( m_paShaderList[i]->LocProjMatrix(),  1, GL_FALSE, (float *)pProj->rc );

		glUniform3fv( m_paShaderList[i]->LocEyePos(), 1, (float *)&pCam->x );

		glUniform3fv( m_paShaderList[i]->LocLightsDir(),       1, (float *)&pLight->GetDir().x );
		glUniform3fv( m_paShaderList[i]->LocLightsIntensity(), 1, (float *)&pLight->GetIntensity().r );
	}

	s16           sCluster = m_pData->sCluster;
	ClusterData * pCluster = m_pData->pCluster;

	for( i=0; i<(u32)sCluster; i++ ) {

		m_paMaterialTbl[pCluster[i].sMaterialIndex]->Fill();
		
		// `
		glBindVertexArray( m_pVAO[i] );
		glBindBuffer( GL_ARRAY_BUFFER, m_pVBO[i] );
		glBindBuffer( GL_ELEMENT_ARRAY_BUFFER, m_pEBO[i] );

		glDrawElements( GL_TRIANGLES, pCluster[i].nIdxBuffer, GL_UNSIGNED_INT, 0 );
	}
	glBindVertexArray( 0 );
	glBindBuffer( GL_ARRAY_BUFFER, 0 );
	glBindBuffer( GL_ELEMENT_ARRAY_BUFFER, 0 );
}
