/** @file texture.cpp
 *  @brief TextureNX\[X
 *
 *  @author yasufuku
 *  @date 2011/02
 *  @version 0.10
 *
 *  Copyright(C) 2004-2011 Kensuke Yasufuku
 */
#include <string.h>
#include "mathlib.h"
#include "texture.h"

//====================================================================
// X^eBbNo
//====================================================================

//====================================================================
// TextureNXo֐`
//====================================================================
//--------------------------------------------------------------
// RXgN^
//--------------------------------------------------------------
Texture::Texture()
{
	m_HeaderTbl.sTexNum  = 0;
	m_HeaderTbl.pHeaderTbl = NULL;
	
	m_pImageTbl = NULL;
	m_pIdTbl    = NULL;
	m_unSize    = 0;
}

Texture::Texture( const Texture & rhs )
{
	m_HeaderTbl.sTexNum  = 0;
	m_HeaderTbl.pHeaderTbl = NULL;
	
	m_pImageTbl = NULL;
	m_pIdTbl    = NULL;
	m_unSize    = 0;
}

//--------------------------------------------------------------
// fXgN^
//--------------------------------------------------------------
Texture::~Texture()
{
}

//--------------------------------------------------------------
// [h
//--------------------------------------------------------------
bool Texture::Load( const HeaderTbl & header_tbl )
{	
	int i, j;
	
	m_HeaderTbl = header_tbl;
	
	u16         sTexNum    = header_tbl.sTexNum;
	const TexHeader * pHeaderTbl = header_tbl.pHeaderTbl;
	
	m_pImageTbl = (u8 ***)NEW u8 **[sTexNum];
	m_pIdTbl    = (u32 *)NEW u32[sTexNum];

	for( i=0; i<sTexNum; i++ ) {
		m_pIdTbl[i] = 0;
	}
	m_unSize = 0;
	
	glEnable( GL_TEXTURE_2D );
	glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
	glGenTextures( sTexNum, m_pIdTbl );
	
	u32 TextureUnit =  GL_TEXTURE0;

	for( i=0; i<sTexNum; i++ ) {

		//printf("DEBUG0[%d](%d)%x\n", i, m_pIdTbl[i], this );
		glActiveTexture( TextureUnit );
		glBindTexture( GL_TEXTURE_2D, m_pIdTbl[i] );
		
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, pHeaderTbl[i].nWrapS );
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, pHeaderTbl[i].nWrapT );
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, pHeaderTbl[i].nMinFilter );
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, pHeaderTbl[i].nMagFilter );
		

		int nMipMap = (int)pHeaderTbl[i].unMipMapLevel+1;
		
		m_pImageTbl[i] = (u8 **)NEW u8 *[nMipMap];
		for( j=0; j<nMipMap; j++ ) {
			// t@CC[Wf[^̃[h()
			m_pImageTbl[i][j] = LoadImageData( &pHeaderTbl[i].pImage[j], &m_unSize );

			u32 internal_format = pHeaderTbl[i].pImage[j].unFormat;
			if( pHeaderTbl[i].pImage[j].unFormat == GL_RGBA ) {
				internal_format = GL_RGBA8;
			}
			glTexImage2D( GL_TEXTURE_2D, j,
						  internal_format,
						  pHeaderTbl[i].pImage[j].usWidth,
						  pHeaderTbl[i].pImage[j].usHeight,
						  0,
						  pHeaderTbl[i].pImage[j].unFormat,
						  GL_UNSIGNED_BYTE,
						  m_pImageTbl[i][j] );
		}
		if( ++TextureUnit > GL_TEXTURE31 ) assert(false);
	}

	return true;
}

//--------------------------------------------------------------
// A[h
//--------------------------------------------------------------
void Texture::Unload()
{
	u16         sTexNum    = m_HeaderTbl.sTexNum;
	const TexHeader * pHeaderTbl = m_HeaderTbl.pHeaderTbl;

	// C[We[ủ
	if( m_pImageTbl ) {
		for( int i=0; i<sTexNum; i++ ) {

			int nMipMap = (int)pHeaderTbl[i].unMipMapLevel+1;
		
			for( int j=0; j<nMipMap; j++ ) {
				delete [] m_pImageTbl[i][j];
			}
			delete [] m_pImageTbl[i];
		}
		delete [] m_pImageTbl;		
		m_pImageTbl = NULL;
	}

	// IDe[ủ
	if( m_pIdTbl ) {
		glDeleteTextures( sTexNum, m_pIdTbl );
		delete [] m_pIdTbl;
	}

	m_unSize = 0;
}

//--------------------------------------------------------------
// FILL
//--------------------------------------------------------------
void Texture::Fill( GLuint diffuse, GLuint light )
{
	// 1
	if( m_pIdTbl ) {
		if( diffuse >= 0 ) {
			//printf("DEBUG2(%d)[%x]%x\n", m_pIdTbl[0], m_pIdTbl, this );
			glActiveTexture( GL_TEXTURE0 );
			glBindTexture(GL_TEXTURE_2D , m_pIdTbl[0] );
			glUniform1i( diffuse, 0 );
		}
		
		// 2ڈȍ~̃}`eNX`Ή
		if( m_HeaderTbl.sTexNum > 1 && light >= 0 ) {
			glActiveTexture( GL_TEXTURE1 );
			glBindTexture(GL_TEXTURE_2D , m_pIdTbl[1] );
			glUniform1i( light, 1 );
		}
		
	}
}

//--------------------------------------------------------------
// t@C(.tga)eNX`[h()
//--------------------------------------------------------------
/**
 *	Ƃ肠Ctgat@Cǂݍ݂Ή
 */
u8 * Texture::LoadImageData( const TexImage * pImage, u32 * pLoadSize )
{
	u8 * ret = NULL;
	switch( pImage->unFormat )
	{
	  case GL_ALPHA:
		ret =  Texture::LoadAlphaImageData( pImage, pLoadSize );
		break;
		
	  case GL_RGBA:
	  default:
		ret = Texture::LoadRGBAImageData( pImage, pLoadSize );
		break;
	}
	return ret;
}

//--------------------------------------------------------------
// t@C(.tga)eNX`[h()
//--------------------------------------------------------------
u8 * Texture::LoadRGBAImageData( const TexImage * pImage, u32 * pLoadSize )
{ 
	FILE *fp;
	
	char * fname  = pImage->pszFileName;
	int    h, w;
	u16    height = pImage->usHeight;
	u16    width  = pImage->usWidth;
	
	/* texture file open */
	if( ( fp = fopen( fname, "rb" ) ) == NULL ){
		PRINTF( "texture file[%s] cannot open\n", fname );
		return NULL;
	}
	PRINTF("Load(%s)\n", fname);

	// C[Ẅm
	int  image_size = height*width*4;
	u8 * image = NEW u8[image_size];
	if( pLoadSize ) *pLoadSize += image_size;

	u16 h_max, w_max;
	fseek( fp, 0x0c, SEEK_SET );
	fread( &w_max, sizeof(u16), 1, fp );
	fread( &h_max, sizeof(u16), 1, fp );


	char bpp = fgetc(fp);
	bool bAlpha = false;
	
	if( bpp == 32 ) {
		bAlpha = true;
	}
	
	fseek( fp, 0x01, SEEK_CUR );
	
	for( h=0; h<height; h++ ){
		for( w=0; w<width; w++ ){

			if( h < h_max && w < w_max ) {
				image[ 4*width*h + 4*w + 2 ]=fgetc(fp);/* B */
				image[ 4*width*h + 4*w + 1 ]=fgetc(fp);/* G */
				image[ 4*width*h + 4*w + 0 ]=fgetc(fp);/* R */

				if( bAlpha ) {
					image[ 4*width*h + 4*w + 3 ]=fgetc(fp);/* alpha */
				} else {
					image[ 4*width*h + 4*w + 3 ]=255;/* alpha */
				}
			} else {
				image[ 4*width*h + 4*w + 3 ]=255;
				image[ 4*width*h + 4*w + 2 ]=255;
				image[ 4*width*h + 4*w + 1 ]=255;
				image[ 4*width*h + 4*w + 0 ]=255;

			}
		}
	}
	fclose(fp);

	return image;
}

//--------------------------------------------------------------
// t@C(.tga)eNX`[h()
//--------------------------------------------------------------
u8 * Texture::LoadAlphaImageData( const TexImage * pImage, u32 * pLoadSize )
{ 
	FILE *fp;
	
	char * fname  = pImage->pszFileName;
	int    h, w;
	u16    height = pImage->usHeight;
	u16    width  = pImage->usWidth;
	
	/* texture file open */
	if( ( fp = fopen( fname, "rb" ) ) == NULL ){
		PRINTF( "alpha texture file[%s] cannot open\n", fname );
		return NULL;
	}

	// C[Ẅm
	int  image_size = height*width;
	u8 * image = NEW u8[image_size];
	if( pLoadSize ) *pLoadSize += image_size;

	u16 h_max, w_max;
	fseek( fp, 0x0c, SEEK_SET );
	fread( &w_max, sizeof(u16), 1, fp );
	fread( &h_max, sizeof(u16), 1, fp );
	
	char bpp = fgetc(fp);
	bool bAlpha = false;
	
	if( bpp == 32 ) {
		bAlpha = true;
	}
	
	fseek( fp, 0x01, SEEK_CUR );
	
	for( h=0; h<height; h++ ){
		for( w=0; w<width; w++ ){

			if( h < h_max && w < w_max ) {
				image[ width*h + w]=255-fgetc(fp);/* B */
				fgetc(fp);/* G */
				fgetc(fp);/* R */

				if( bAlpha ) {
					/*image[width*h + w]=*/fgetc(fp);/* alpha */
				}
			} else {
				image[ width*h + w] = 255;
			}
		}
	}
	fclose(fp);

	return image;
}

