/** @file mathlib.h
 *  @brief wCuCvg
 *
 *  @author yasufuku
 *  @date 2011/02
 *  @version 0.20
 *
 *  Copyright(C) 2004-2011 Kensuke Yasufuku
 */
#include "mathlib.h"
#include "vector.h"

#define LOOP_ROUND	(float)(1<<20)
#define LOOP_MASK	((1<<20)-1)


//---------------------------------------------------------------
// NH[^jIRs[
//---------------------------------------------------------------
void CopyTRS( int nBone, TRS * pTrsDst, TRS * pTrsSrc )
{
	u32 * dst = (u32 *)pTrsDst;
	u32 * src = (u32 *)pTrsSrc;
	int   n   = nBone * 10;
	
	while( n-- ) *dst++ = *src++;
}

//---------------------------------------------------------------
// NH[^jIʐ`
//---------------------------------------------------------------
void SlerpQuat( int   nBone,
			    TRS   * pDst,
			    TRS   * pKeyA,
			    TRS   * pKeyB,
			    float t )
{
	float  c[4];
	float  cosom;

	while( nBone-- ) {

		cosom = _DotProduct4( pKeyA->quat, pKeyB->quat );
		if( cosom < 0.0f ) {
			_ScalarVector4( c, pKeyB->quat, -1.0f );
			cosom = -cosom;
		} else {
			_CopyVector4( c, pKeyB->quat );
		}

		if( (1.0f - cosom ) > DELTA ) {
			float omega, sinom, t0, t1;

			omega = acosf(cosom);
			sinom = 1.0f/sinf(omega);
			t0 = sinf( (1.0f - t ) * omega ) * sinom;
			t1 = sinf( t * omega ) * sinom;
			_T2LerpVector4( pDst->quat, pKeyA->quat, c, t0, t1 );

		} else {
			_LerpVector4( pDst->quat, c, pKeyA->quat, t );
		}

		pDst++; pKeyA++; pKeyB++;
	}
}

//---------------------------------------------------------------
// Transf[^̐`
//---------------------------------------------------------------
void LerpTrans( int   nBone,
			    TRS   * pDst,
			    TRS   * pKeyA,
			    TRS   * pKeyB,
			    float t )
{
	while( nBone-- ) {
		_LerpVector3( pDst->trans, pKeyB->trans, pKeyA->trans, t );
		pDst++; pKeyA++; pKeyB++;
	}
}

//---------------------------------------------------------------
// Scalef[^̐`
//---------------------------------------------------------------
void LerpScale( int   nBone,
			    TRS   * pDst,
			    TRS   * pKeyA,
			    TRS   * pKeyB,
			    float t )
{
	while( nBone-- ) {
		_LerpVector3( pDst->scale, pKeyB->scale, pKeyA->scale, t );
		pDst++; pKeyA++; pKeyB++;
	}
}



