/** @file matrix.cpp
 *  @brief MatrixNXCvg
 *
 *  @author yasufuku
 *  @date 2011/02
 *  @version 0.06
 *
 *  Copyright(C) 2004-2011 Kensuke Yasufuku
 */
#include "matrix.h"
#include <math.h>

//====================================================================
// MatrixNX o֐`
//====================================================================
//---------------------------------------------------------------
// RXgN^
//---------------------------------------------------------------
Matrix::Matrix()
{
	Unit();
}

Matrix::Matrix( const Matrix & m )
{
	const float * src = &m.rc[0][0];
	float * dst = &rc[0][0];
	for( int i=0; i<16; i++ ) {
		*dst++ = *src++;
	}
}

Matrix::Matrix( const float m[4][4] )
{
	for( int i=0; i<4; i++ ) {
		for( int j=0; j<4; j++ ) {
			rc[i][j] = m[i][j];
		}
	}
}


//---------------------------------------------------------------
// fXgN^
//---------------------------------------------------------------
Matrix::~Matrix()
{
}

//---------------------------------------------------------------
// vf
//---------------------------------------------------------------
void Matrix::Set( const float m[4][4] )
{
	for( int i=0; i<4; i++ ) {
		for( int j=0; j<4; j++ ) {
			rc[i][j] = m[i][j];
		}
	}
}

//---------------------------------------------------------------
// = }gbNX
//---------------------------------------------------------------
Matrix & Matrix::operator = ( const Matrix & m )
{
	const float * src = &m.rc[0][0];
	float * dst = &rc[0][0];
	for( int i=0; i<16; i++ ) {
		*dst++ = *src++;
	}
	return *this;
}

//---------------------------------------------------------------
// }gbNX~XJ[
//---------------------------------------------------------------
Matrix & Matrix::operator *= ( float scalar )
{
	float * p = &rc[0][0];
	for( int i=0; i<16; i++ ) {
		*p++ = *p++ * scalar;
	}
	return *this;
}

//---------------------------------------------------------------
// }gbNXXJ[
//---------------------------------------------------------------
Matrix & Matrix::operator /= ( float scalar )
{
	float * p = &rc[0][0];
	for( int i=0; i<16; i++ ) {
		*p++ = *p++ / scalar;
	}
	return *this;
}

//---------------------------------------------------------------
// }gbNX~}gbNX
//---------------------------------------------------------------
Matrix & Matrix::operator *= ( const Matrix & m )
{
	int r, c;
	float f;
	Matrix t;

	for( r=0; r<4; r++ ) {
		for( c=0; c<4; c++ ) {
			f =  rc[0][r] * m.rc[c][0];
			f += rc[1][r] * m.rc[c][1];
			f += rc[2][r] * m.rc[c][2];
			f += rc[3][r] * m.rc[c][3];
			t.rc[c][r] = f;
		}
	}

	*this = t;
    return *this;
}

//---------------------------------------------------------------
// }gbNX{}gbNX
//---------------------------------------------------------------
Matrix & Matrix::operator += ( const Matrix & m )
{
	const float * src = &m.rc[0][0];
	float * dst = &rc[0][0];
	for( int i=0; i<16; i++ ) {
		*dst++ += *src++;
	}
	return *this;
}

//---------------------------------------------------------------
// }gbNX|}gbNX
//---------------------------------------------------------------
Matrix & Matrix::operator -= ( const Matrix & m )
{
	const float * src = &m.rc[0][0];
	float * dst = &rc[0][0];
	for( int i=0; i<16; i++ ) {
		*dst++ -= *src++;
	}
	return *this;
}

//---------------------------------------------------------------
// }gbNX~XJ[
//---------------------------------------------------------------
Matrix Matrix::operator * ( float scalar ) const
{
	Matrix ret;
	const float * src = &rc[0][0];
	float * dst = &ret.rc[0][0];
	for( int i=0; i<16; i++ ) {
		*dst++ = *src++ * scalar;
	}
	return ret;
}

//---------------------------------------------------------------
// }gbNXXJ[
//---------------------------------------------------------------
Matrix Matrix::operator / ( float scalar ) const
{
	Matrix ret;
	const float * src = &rc[0][0];
	float * dst = &ret.rc[0][0];
	for( int i=0; i<16; i++ ) {
		*dst++ = *src++ / scalar;
	}
	return ret;
}

//---------------------------------------------------------------
// }gbNX~}gbNX
//---------------------------------------------------------------
Matrix Matrix::operator * ( const Matrix & m ) const
{
	Matrix ret( *this );
	ret *= m;
	return ret;
}

//---------------------------------------------------------------
// }gbNX{}gbNX
//---------------------------------------------------------------
Matrix Matrix::operator + ( const Matrix & m ) const
{
	Matrix ret( *this );
	ret += m;
	return ret;
}

//---------------------------------------------------------------
// }gbNX~}gbNX
//---------------------------------------------------------------
Matrix Matrix::operator - ( const Matrix & m ) const
{
	Matrix ret( *this );
	ret -= m;
	return ret;
}

//---------------------------------------------------------------
// }gbNX~xNg
//---------------------------------------------------------------
Vector Matrix::operator * ( const Vector & v ) const
{
	Vector ret;

	ret.x = v.x*rc[0][0] + v.y*rc[1][0] + v.z*rc[2][0] + v.w*rc[3][0];
	ret.y = v.x*rc[0][1] + v.y*rc[1][1] + v.z*rc[2][1] + v.w*rc[3][1];
	ret.z = v.x*rc[0][2] + v.y*rc[1][2] + v.z*rc[2][2] + v.w*rc[3][2];
	ret.w = v.x*rc[0][3] + v.y*rc[1][3] + v.z*rc[2][3] + v.w*rc[3][3];

	return ret;
}

//---------------------------------------------------------------
// Pʍs
//---------------------------------------------------------------
Matrix & Matrix::Unit()
{
	rc[0][0] = 1.0f; rc[0][1] = 0.0f; rc[0][2] = 0.0f; rc[0][3] = 0.0f;
	rc[1][0] = 0.0f; rc[1][1] = 1.0f; rc[1][2] = 0.0f; rc[1][3] = 0.0f;
	rc[2][0] = 0.0f; rc[2][1] = 0.0f; rc[2][2] = 1.0f; rc[2][3] = 0.0f;
	rc[3][0] = 0.0f; rc[3][1] = 0.0f; rc[3][2] = 0.0f; rc[3][3] = 1.0f;

	return *this;
}

//---------------------------------------------------------------
// s
//---------------------------------------------------------------
Matrix & Matrix::Zero()
{
	float * p = &rc[0][0];
	for( int i=0; i<16; i++ ) {
		*p++ = 0.0f;
	}
	return *this;
}
	
//---------------------------------------------------------------
// ]us
//---------------------------------------------------------------
Matrix & Matrix::Transpose()
{
	int c, r;
	float t;

	for( c=0; c<4; c++) {
		for( r=c+1; r<4; r++) {
			t = rc[c][r];
			rc[c][r] = rc[r][c];
			rc[r][c] = t;
		} 
	}
	return *this;
}

//---------------------------------------------------------------
// ts
//---------------------------------------------------------------
Matrix & Matrix::Invert()
{
	Matrix a(*this);
	Matrix b;

	u32 r, c;
	u32 cc;
	u32 rowMax;
	u32 row;
	float tmp;

	for( c=0; c<4; c++ ) {
		// Find the row with max value in this column
		rowMax = c;
		for( r=c+1; r<4; r++ ) {
			if( fabs(a.rc[c][r]) > fabs(a.rc[c][rowMax]) ) {
				rowMax = r;
			}
		}

		// If the max value here is 0, we can't invert.  Return identity.
		//if (a.rc[rowMax][c] == 0.0F) {
		//	return Unit();
		//}

		// Swap row "rowMax" with row "c"
		for( cc=0; cc<4; cc++ ) {
			tmp = a.rc[cc][c];
			a.rc[cc][c] = a.rc[cc][rowMax];
			a.rc[cc][rowMax] = tmp;
			tmp = b.rc[cc][c];
			b.rc[cc][c] = b.rc[cc][rowMax];
			b.rc[cc][rowMax] = tmp;
		}

		// Now everything we do is on row "c".
		// Set the max cell to 1 by dividing the entire row by that value
		tmp = a.rc[c][c];
		for( cc=0; cc<4; cc++ ) {
			a.rc[cc][c] /= tmp;
			b.rc[cc][c] /= tmp;
		}

		// Now do the other rows, so that this column only has a 1 and 0's
		for( row=0; row<4; row++ ) {
			if (row != c) {
				tmp = a.rc[c][row];
				for( cc=0; cc<4; cc++ ) {
					a.rc[cc][row] -= a.rc[cc][c] * tmp;
					b.rc[cc][row] -= b.rc[cc][c] * tmp;
				}
			}
		}
	}
	*this = b;
	return *this;
}

//---------------------------------------------------------------
// ]us
//---------------------------------------------------------------
/**
 * ]us擾@gȂ
 * @retval: ]us
 */
Matrix Matrix::TransposedMatrix() const
{
	Matrix ret(*this);
	return ret.Transpose();
}


//---------------------------------------------------------------
// ts
//---------------------------------------------------------------
/**
 * ts擾@gȂ
 * @retval: ts
 */
Matrix Matrix::InverseMatrix() const
{
	Matrix ret(*this);
	return ret.Invert();
}

//---------------------------------------------------------------
// sړ(E|)
//---------------------------------------------------------------
Matrix & Matrix::TransR( const Vector & t )
{
	return TransR( t.x, t.y, t.z );
}

Matrix & Matrix::TransR( float x, float y, float z )
{
	float t[4];
	
	t[0] = x*rc[0][0] + y*rc[1][0] + z*rc[2][0] + rc[3][0];
    t[1] = x*rc[0][1] + y*rc[1][1] + z*rc[2][1] + rc[3][1];
    t[2] = x*rc[0][2] + y*rc[1][2] + z*rc[2][2] + rc[3][2];
//  t[3] = x*rc[0][3] + y*rc[1][3] + z*rc[2][3] + rc[3][3];

    rc[3][0] = t[0];
    rc[3][1] = t[1];
    rc[3][2] = t[2];
//  rc[3][3] = t[3];

	return *this;
}

//---------------------------------------------------------------
// sړ(|)
//---------------------------------------------------------------
Matrix & Matrix::TransL( const Vector & t )
{
	return TransL( t.x, t.y, t.z );
}

Matrix & Matrix::TransL( float x, float y, float z )
{
	rc[3][0] += x;
	rc[3][1] += y;
	rc[3][2] += z;
	return *this;
}

//---------------------------------------------------------------
// sړ(|)
//---------------------------------------------------------------
Matrix & Matrix::SetTrans( const Vector & t )
{
	return SetTrans( t.x, t.y, t.z );
}

Matrix & Matrix::SetTrans( float x, float y, float z )
{
	rc[3][0] = x;
	rc[3][1] = y;
	rc[3][2] = z;
	return *this;
}

//---------------------------------------------------------------
// X]̃}gbNX|
//---------------------------------------------------------------
Matrix & Matrix::RotX( float x )
{
	Matrix t;

	float cosA = cosf(x);
	float sinA = sinf(x);
	
	// axisX
	t.rc[0][0] = 1.0f; t.rc[1][0] = 0.0f; t.rc[2][0] = 0.0f;
	// axisY
	t.rc[0][1] = 0.0f; t.rc[1][1] = cosA; t.rc[2][1] =-sinA;
	// axisZ
	t.rc[0][2] = 0.0f; t.rc[1][2] = sinA; t.rc[2][2] = cosA;

	
	t.rc[3][0] = 0.0f; t.rc[3][1] = 0.0f; t.rc[3][2] = 0.0f;
	t.rc[0][3] = 0.0f; t.rc[1][3] = 0.0f; t.rc[2][3] = 0.0f;
	t.rc[3][3] = 1.0f;

	*this = t * (*this);
	return *this;
}

//---------------------------------------------------------------
// Y]̃}gbNX|
//---------------------------------------------------------------
Matrix & Matrix::RotY( float y )
{
	Matrix t;

	float cosA = cosf(y);
	float sinA = sinf(y);
	
	// axisX
	t.rc[0][0] = cosA; t.rc[1][0] = 0.0f; t.rc[2][0] = sinA;
	// axisY
	t.rc[0][1] = 0.0f; t.rc[1][1] = 1.0f; t.rc[2][1] = 0.0f;
	// axisZ
	t.rc[0][2] =-sinA; t.rc[1][2] = 0.0f; t.rc[2][2] = cosA;

	
	t.rc[3][0] = 0.0f; t.rc[3][1] = 0.0f; t.rc[3][2] = 0.0f;
	t.rc[0][3] = 0.0f; t.rc[1][3] = 0.0f; t.rc[2][3] = 0.0f;
	t.rc[3][3] = 1.0f;

	*this = t * (*this);
	return *this;
}

//---------------------------------------------------------------
// Z]̃}gbNX|
//---------------------------------------------------------------
Matrix & Matrix::RotZ( float z )
{
	Matrix t;

	float cosA = cosf(z);
	float sinA = sinf(z);
	
	// axisX
	t.rc[0][0] = cosA; t.rc[1][0] =-sinA; t.rc[2][0] = 0.0f;
	// axisY
	t.rc[0][1] = sinA; t.rc[1][1] = cosA; t.rc[2][1] = 0.0f;
	// axisZ
	t.rc[0][2] = 0.0f; t.rc[1][2] = 0.0f; t.rc[2][2] = 1.0f;

	
	t.rc[3][0] = 0.0f; t.rc[3][1] = 0.0f; t.rc[3][2] = 0.0f;
	t.rc[0][3] = 0.0f; t.rc[1][3] = 0.0f; t.rc[2][3] = 0.0f;
	t.rc[3][3] = 1.0f;

	*this = t * (*this);
	return *this;
}

//---------------------------------------------------------------
// XYZ̏Ԃŉ]}gbNX|
//---------------------------------------------------------------
Matrix & Matrix::RotXYZ( const Vector & v )
{
	return RotXYZ( v.x, v.y, v.z );
}

Matrix & Matrix::RotXYZ( float x, float y, float z )
{
	RotZ(z);
	RotY(y);
	RotX(x);

	return * this;
}

//---------------------------------------------------------------
// ZYX̏Ԃŉ]}gbNX|
//---------------------------------------------------------------
Matrix & Matrix::RotZYX( const Vector & v )
{
	return RotZYX( v.x, v.y, v.z );
}

Matrix & Matrix::RotZYX( float x, float y, float z )
{
	RotX(x);
	RotY(y);
	RotZ(z);

	return * this;
}

//---------------------------------------------------------------
// XP[}gbNX|
//---------------------------------------------------------------
Matrix & Matrix::ScaleL( const Vector & v )
{
	return ScaleL( v.x, v.y, v.z );
}

Matrix & Matrix::ScaleL( float x, float y, float z )
{
	rc[0][0] *= x;
	rc[1][0] *= x;
	rc[2][0] *= x;
	rc[3][0] *= x;

	rc[0][1] *= y;
	rc[1][1] *= y;
	rc[2][1] *= y;
	rc[3][1] *= y;

	rc[0][2] *= z;
	rc[1][2] *= z;
	rc[2][2] *= z;
	rc[3][2] *= z;



	return *this;
}

//---------------------------------------------------------------
// XP[}gbNXE|
//---------------------------------------------------------------
Matrix & Matrix::ScaleR( const Vector & v )
{
	return ScaleR( v.x, v.y, v.z );
}

Matrix & Matrix::ScaleR( float x, float y, float z )
{
	rc[0][0] *= x;
	rc[0][1] *= x;
	rc[0][2] *= x;
	//rc[0][3] *= x;

	rc[1][0] *= y;
	rc[1][1] *= y;
	rc[1][2] *= y;
	//rc[1][3] *= y;

	rc[2][0] *= z;
	rc[2][1] *= z;
	rc[2][2] *= z;
	//rc[2][3] *= z;

	return *this;
}

//---------------------------------------------------------------
// Lookat
//---------------------------------------------------------------
Matrix & Matrix::Lookat( const Vector & t, const Vector & at, const Vector & up )
{
	Vector az  = (t-at).Normalize();
	Vector tup = up;
	float naiseki = az*tup;
	if( IsNearlyZero(fabs(naiseki)-1.0f) ) {
		tup = Vector(0.0f, 0.0f, -1.0f);
		naiseki = az*tup;
	}
	Vector ay = (tup-naiseki*az).Normalize();
	Vector ax = CrossProduct(ay,az);
	
	Unit();
	// axisX
	rc[0][0] = ax.x;
	rc[1][0] = ax.y;
	rc[2][0] = ax.z;
	
	// axisY
	rc[0][1] = ay.x;
	rc[1][1] = ay.y;
	rc[2][1] = ay.z;
	
	// axisZ
	rc[0][2] = az.x;
	rc[1][2] = az.y;
	rc[2][2] = az.z;
	
	TransR( -t.x, -t.y, -t.z );

	return *this;
}


//---------------------------------------------------------------
// ˉe}gbNX( Perspective : EΏ̂ȓ@̍s )
//---------------------------------------------------------------
Matrix & Matrix::Perspective( float fFovy, float fAspect, float fNear, float fFar )
{
	float range  = fNear * tanf( DegToRad(fFovy*0.5f) );
	float left   = -range * fAspect;
	float right  =  range * fAspect;
	float bottom = -range;
	float top    =  range;
	

	rc[0][0] = (2.0f * fNear)/(right-left);
	rc[0][1] = 0.0f;
	rc[0][2] = 0.0f;
	rc[0][3] = 0.0f;

	rc[1][0] = 0.0f;
	rc[1][1] = (2.0f * fNear)/(top-bottom);
	rc[1][2] = 0.0f;
	rc[1][3] = 0.0f;

	rc[2][0] = 0.0f;
	rc[2][1] = 0.0f;
	rc[2][2] = -(fFar+fNear)/(fFar-fNear);
	rc[2][3] = -1.0f;

	rc[3][0] = 0.0f;
	rc[3][1] = 0.0f;
	rc[3][2] = - (2.0f * fFar * fNear)/(fFar - fNear);
	rc[3][3] = 0.0f;

	return *this;
}

//---------------------------------------------------------------
// ˉe}gbNX( Ortho : ˉeɂ镽sȎ̐ς̍s )
//---------------------------------------------------------------
Matrix & Matrix::Ortho( float left,   float right,
						float bottom, float top,
						float fnear,   float far )
{
	float tx = (right+left)/(right-left);
	float ty = (top+bottom)/(top-bottom);
	float tz = (far+fnear)/(far-fnear);

	rc[0][0] = 2.0f/(right-left);
	rc[0][1] = 0.0f;
	rc[0][2] = 0.0f;
	rc[0][3] = 0.0f;

	rc[1][0] = 0.0f;
	rc[1][1] = 2.0f/(top-bottom);
	rc[1][2] = 0.0f;
	rc[1][3] = 0.0f;

	rc[2][0] = 0.0f;
	rc[2][1] = 0.0f;
	rc[2][2] = 0.0f;//-2.0f/(far-fnear);
	rc[2][3] = 0.0f;

	rc[3][0] = tx;
	rc[3][1] = ty;
	rc[3][2] = 0.0f;//tz;
	rc[3][3] = 1.0f;

	//Transpose();
	
	return *this;
}

void Matrix::Print()
{
	printf( "|%f %f %f %f|\n", rc[0][0], rc[0][1], rc[0][2], rc[0][3] );
	printf( "|%f %f %f %f|\n", rc[1][0], rc[1][1], rc[1][2], rc[1][3] );
	printf( "|%f %f %f %f|\n", rc[2][0], rc[2][1], rc[2][2], rc[2][3] );
	printf( "|%f %f %f %f|\n", rc[3][0], rc[3][1], rc[3][2], rc[3][3] );
}
