/** @file keyboard.cpp
 *  @brief KeyboardNX\[X
 *
 *  @author yasufuku
 *  @date 2011/02
 *  @version 0.10
 *
 *  Copyright(C) 2004-2011 Kensuke Yasufuku
 */
#include "keyboard.h"

//====================================================================
// O[o֐(GLUTo^p)
//====================================================================
static void PressKey( u8 key, int x, int y )
{
	Keyboard::GetSelf()->CallBackPressKey( key, x, y );

	int mod = glutGetModifiers();
	Keyboard::GetSelf()->CallBackCheckModifiers(mod);
}

static void UnpressKey( u8 key, int x, int y )
{
	Keyboard::GetSelf()->CallBackUnpressKey( key, x, y );

	int mod = glutGetModifiers();
	Keyboard::GetSelf()->CallBackCheckModifiers(mod);
}

static void PressSpecialKey( int key, int x, int y )
{
	Keyboard::GetSelf()->CallBackPressSpecialKey( key, x, y );

	int mod = glutGetModifiers();
	Keyboard::GetSelf()->CallBackCheckModifiers(mod);
}

static void UnpressSpecialKey( int key, int x, int y )
{
	Keyboard::GetSelf()->CallBackUnpressSpecialKey( key, x, y );

	//int mod = glutGetModifiers();
	//Keyboard::GetSelf()->CallBackCheckModifiers(mod);
}


//====================================================================
// statico
//====================================================================
Keyboard * Keyboard::s_pSelf             = NULL;
u32        Keyboard::s_unInputKey        = 0;
u32        Keyboard::s_unInputSpecialKey = 0;

//====================================================================
// HumanNXo֐`
//====================================================================
//---------------------------------------------------------------
// 
//---------------------------------------------------------------
Keyboard * Keyboard::Create( Task * pParent )
{
	if( s_pSelf == NULL ) {
		s_pSelf = NEW Keyboard;
		s_pSelf->Init( pParent, 1 );
	}
	return s_pSelf;
}

//---------------------------------------------------------------
// RXgN^
//---------------------------------------------------------------
Keyboard::Keyboard()
{
	SetName("Keyboard");
	m_unPress = 0;
	m_unTrigger = 0;
	m_unSPress = 0;
	m_unSTrigger = 0;
}

//---------------------------------------------------------------
// fXgN^
//---------------------------------------------------------------
Keyboard::~Keyboard()
{
	s_pSelf = NULL;
}

//---------------------------------------------------------------
// 
//---------------------------------------------------------------
int Keyboard::OnInit()
{
	// L[{[h֐ݒ
	glutKeyboardFunc(PressKey);
	glutKeyboardUpFunc(UnpressKey);
	glutSpecialFunc(PressSpecialKey);
	glutSpecialUpFunc(UnpressSpecialKey);

	return TR_OK;
}
//---------------------------------------------------------------
// C[v
//---------------------------------------------------------------
int Keyboard::OnMain()
{	
	m_unTrigger = (int) ( s_unInputKey & ( s_unInputKey ^ m_unPress ) );
	m_unPress   = s_unInputKey;

	m_unSTrigger = (int) ( s_unInputSpecialKey & ( s_unInputSpecialKey ^ m_unSPress ) );
	m_unSPress   = s_unInputSpecialKey;
	
	return TR_OK;
}

//---------------------------------------------------------------
// L[ꑱĂ邩
//---------------------------------------------------------------
bool Keyboard::Press( eKey key )
{
	return s_pSelf->m_unPress & key ? true : false;
}

//---------------------------------------------------------------
// L[Ă邩
//---------------------------------------------------------------
bool Keyboard::Trigger( eKey key )
{
	return s_pSelf->m_unTrigger & key ? true : false;
}

//---------------------------------------------------------------
// L[ꑱĂ邩
//---------------------------------------------------------------
bool Keyboard::Press( eSKey skey )
{
	return s_pSelf->m_unSPress & skey ? true : false;
}

//---------------------------------------------------------------
// L[Ă邩
//---------------------------------------------------------------
bool Keyboard::Trigger( eSKey skey )
{
	return s_pSelf->m_unSTrigger & skey ? true : false;
}

//---------------------------------------------------------------
// GLUT̃R[obN֐
//---------------------------------------------------------------
void Keyboard::CallBackPressKey( u8 key, int x, int y )
{
	//printf("Key:%d(%d,%d)\n", key, x, y );
	switch (key) {
		case 'a':
			s_unInputKey |= KEY_A;
			break;
		case 'b':
			s_unInputKey |= KEY_B;
			break;
		case 'c':
			s_unInputKey |= KEY_C;
			break;
		case 'd':
			s_unInputKey |= KEY_D;
			break;
		case 'e':
			s_unInputKey |= KEY_E;
			break;
		case 'f':
			s_unInputKey |= KEY_F;
			break;
		case 'g':
			s_unInputKey |= KEY_G;
			break;
		case 'h':
			s_unInputKey |= KEY_H;
			break;
		case 'i':
			s_unInputKey |= KEY_I;
			break;
		case 'j':
			s_unInputKey |= KEY_J;
			break;
		case 'k':
			s_unInputKey |= KEY_K;
			break;
		case 'l':
			s_unInputKey |= KEY_L;
			break;
		case 'm':
			s_unInputKey |= KEY_M;
			break;
		case 'n':
			s_unInputKey |= KEY_N;
			break;
		case 'o':
			s_unInputKey |= KEY_O;
			break;
		case 'p':
			s_unInputKey |= KEY_P;
			break;
		case 'q':
			s_unInputKey |= KEY_Q;
			break;
		case 'r':
			s_unInputKey |= KEY_R;
			break;
		case 's':
			s_unInputKey |= KEY_S;
			break;
		case 't':
			s_unInputKey |= KEY_T;
			break;
		case 'u':
			s_unInputKey |= KEY_U;
			break;
		case 'v':
			s_unInputKey |= KEY_V;
			break;
		case 'w':
			s_unInputKey |= KEY_W;
			break;
		case 'x':
			s_unInputKey |= KEY_X;
			break;
		case 'y':
			s_unInputKey |= KEY_Y;
			break;
		case 'z':
			s_unInputKey |= KEY_Z;
			break;
		case '\015':
			s_unInputKey |= KEY_ENTER;
			break;
		case '\040':
			s_unInputKey |= KEY_SPACE;
			break;
		case '\033':
			s_unInputSpecialKey |= KEY_ESC;
			break;
		default:
			break;
	}
}

//---------------------------------------------------------------
// GLUT̃R[obN֐
//---------------------------------------------------------------
void Keyboard::CallBackUnpressKey( u8 key, int x, int y )
{
	switch (key) {
		case 'a':
			s_unInputKey &= ~KEY_A;
			break;
		case 'b':
			s_unInputKey &= ~KEY_B;
			break;
		case 'c':
			s_unInputKey &= ~KEY_C;
			break;
		case 'd':
			s_unInputKey &= ~KEY_D;
			break;
		case 'e':
			s_unInputKey &= ~KEY_E;
			break;
		case 'f':
			s_unInputKey &= ~KEY_F;
			break;
		case 'g':
			s_unInputKey &= ~KEY_G;
			break;
		case 'h':
			s_unInputKey &= ~KEY_H;
			break;
		case 'i':
			s_unInputKey &= ~KEY_I;
			break;
		case 'j':
			s_unInputKey &= ~KEY_J;
			break;
		case 'k':
			s_unInputKey &= ~KEY_K;
			break;
		case 'l':
			s_unInputKey &= ~KEY_L;
			break;
		case 'm':
			s_unInputKey &= ~KEY_M;
			break;
		case 'n':
			s_unInputKey &= ~KEY_N;
			break;
		case 'o':
			s_unInputKey &= ~KEY_O;
			break;
		case 'p':
			s_unInputKey &= ~KEY_P;
			break;
		case 'q':
			s_unInputKey &= ~KEY_Q;
			break;
		case 'r':
			s_unInputKey &= ~KEY_R;
			break;
		case 's':
			s_unInputKey &= ~KEY_S;
			break;
		case 't':
			s_unInputKey &= ~KEY_T;
			break;
		case 'u':
			s_unInputKey &= ~KEY_U;
			break;
		case 'v':
			s_unInputKey &= ~KEY_V;
			break;
		case 'w':
			s_unInputKey &= ~KEY_W;
			break;
		case 'x':
			s_unInputKey &= ~KEY_X;
			break;
		case 'y':
			s_unInputKey &= ~KEY_Y;
			break;
		case 'z':
			s_unInputKey &= ~KEY_Z;
			break;
		case '\015':
			s_unInputKey &= ~KEY_ENTER;
			break;
		case '\040':
			s_unInputKey &= ~KEY_SPACE;
			break;
		case '\033':
			s_unInputSpecialKey &= ~KEY_ESC;
			break;
		default:
			break;
	}
}

//---------------------------------------------------------------
// GLUT̃R[obN֐
//---------------------------------------------------------------
void Keyboard::CallBackPressSpecialKey( int key, int x, int y )
{
	switch (key) {
		case GLUT_KEY_LEFT:
			s_unInputKey |= KEY_LEFT;
			break;
		case GLUT_KEY_RIGHT:
			s_unInputKey |= KEY_RIGHT;
			break;
		case GLUT_KEY_UP:
			s_unInputKey |= KEY_UP;
			break;
		case GLUT_KEY_DOWN:
			s_unInputKey |= KEY_DOWN;
			break;
		case GLUT_KEY_F1:
			s_unInputSpecialKey  |= KEY_F1;
			break;
	}
}

//---------------------------------------------------------------
// GLUT̃R[obN֐
//---------------------------------------------------------------
void Keyboard::CallBackUnpressSpecialKey( int key, int x, int y )
{
	switch (key) {
		case GLUT_KEY_LEFT:
			s_unInputKey &= ~KEY_LEFT;
			break;
		case GLUT_KEY_RIGHT:
			s_unInputKey &= ~KEY_RIGHT;
			break;
		case GLUT_KEY_UP:
			s_unInputKey &= ~KEY_UP;
			break;
		case GLUT_KEY_DOWN:
			s_unInputKey &= ~KEY_DOWN;
			break;
		case GLUT_KEY_F1:
			s_unInputSpecialKey  &= ~KEY_F1;
			break;
	}
}

//---------------------------------------------------------------
// GLUT̃R[obN֐
//---------------------------------------------------------------
void Keyboard::CallBackCheckModifiers( int mod )
{
	if( mod & GLUT_ACTIVE_SHIFT) {
		s_unInputSpecialKey |= KEY_SHIFT;
	} else {
		s_unInputSpecialKey &= ~KEY_SHIFT;
	}
	if( mod & GLUT_ACTIVE_CTRL) {
		s_unInputSpecialKey |= KEY_CTRL;
	} else {
		s_unInputSpecialKey &= ~KEY_CTRL;
	}
	if( mod & GLUT_ACTIVE_ALT) {
		s_unInputSpecialKey |= KEY_ALT;
	} else {
		s_unInputSpecialKey &= ~KEY_ALT;
	}

	
}
