/** @file mouse.cpp
 *  @brief MouseNX\[X
 *
 *  @author yasufuku
 *  @date 2011/02
 *  @version 0.10
 *
 *  Copyright(C) 2004-2011 Kensuke Yasufuku
 */

//====================================================================
// CN[h
//====================================================================
#include "mouse.h"
#include "sys_config.h"
#include "keyboard.h"
extern SystemConfig sc;

//====================================================================
// X^eBbNo
//====================================================================
Mouse * Mouse::s_pSelf = NULL;

//====================================================================
// O[o֐`
//====================================================================
// ςȂ...
static void clickMouse( int button, int stat, int x, int y )
{
	int mod = glutGetModifiers();
	Keyboard::GetSelf()->CallBackCheckModifiers(mod);
	Mouse::GetSelf()->CallBackClick( button, stat, x, y );
}

static void dragMouse( int x, int y )
{
	Mouse::GetSelf()->CallBackDrag( x, y );
}

static void moveMouse( int x, int y )
{
	Mouse::GetSelf()->CallBackMove( x, y );
}

//====================================================================
// MouseNXo֐`
//====================================================================
//--------------------------------------------------------------
// ֐
//--------------------------------------------------------------
/**
 *	@param	pParent		e^XN(ʏRootTask)
 *	@retval B̃CX^X(Singleton)
 */
Mouse * Mouse::Create( Task * pParent )
{
	if( s_pSelf == NULL ) {
		s_pSelf = NEW Mouse;
		s_pSelf->Init( pParent, 0 );
	}

	return s_pSelf;
}

//--------------------------------------------------------------
// RXgN^
//--------------------------------------------------------------
Mouse::Mouse()
{
	SetName("Mouse");

	m_bDrag = false;
	m_nX = 0;
	m_nY = 0;
	m_nClickX = 0;
	m_nClickY = 0;
	m_unClick = 0;
	m_unDrag  = 0;
	m_nMoveX = 0;
	m_nMoveY = 0;
	m_nPrevMoveX = 0;
	m_nPrevMoveY = 0;
	m_nMoveDX = 0;
	m_nMoveDY = 0;
	m_nDragDX = 0;
	m_nDragDY = 0;
	m_bMove = false;
	m_bFPSMode = false;
}

//--------------------------------------------------------------
// fXgN^
//--------------------------------------------------------------
Mouse::~Mouse()
{
	if( s_pSelf ) {
		s_pSelf = NULL;
	}
}

//--------------------------------------------------------------
// GLUT̃R[obN֐
//--------------------------------------------------------------
void Mouse::CallBackClick( int button, int stat, int x, int y )
{
	u32 prevClick = m_unClick;
	
	switch( button )
	{
	  case GLUT_LEFT_BUTTON:
		if( stat == GLUT_DOWN ) { m_unClick |=  MOUSE_L_BUTTON; }
		if( stat == GLUT_UP   ) { m_unClick &= ~MOUSE_L_BUTTON; }
		break;
		
	  case GLUT_RIGHT_BUTTON:
		if( stat == GLUT_DOWN ) { m_unClick |=  MOUSE_R_BUTTON; }
		if( stat == GLUT_UP   ) { m_unClick &= ~MOUSE_R_BUTTON; }
		break;
	  case GLUT_MIDDLE_BUTTON:
		if( stat == GLUT_DOWN ) { m_unClick |=  MOUSE_M_BUTTON; }
		if( stat == GLUT_UP   ) { m_unClick &= ~MOUSE_M_BUTTON; }
		break;
	  default:
		break;
	}

	if( stat == GLUT_DOWN ) {

		m_nClickX = x;
		m_nClickY = y;
		m_nX      = x;
		m_nY      = y;
		m_nPrevX  = x;
		m_nPrevY  = y;
	}

	// hbO
	m_unDrag = m_unClick;
	//printf("Mouse Click x:%d y:%d(%d, %d)\n", m_nClickX, m_nClickY, m_unClick, m_unDrag );

}

//--------------------------------------------------------------
// GLUT̃R[obN֐
//--------------------------------------------------------------
void Mouse::CallBackDrag( int x, int y )
{
	// hbO
	m_bDrag = TRUE;
	m_unDrag = m_unClick;

	m_nPrevX = m_nX;
	m_nPrevY = m_nY;
	
	m_nX = x;
	m_nY = y;

	//printf("Mouse Drag  x:%d  y:%d\n", x, y );
	//printf("Mouse Drag vx:%d vy:%d\n", m_nX-m_nPrevX, m_nY-m_nPrevY );
}

//--------------------------------------------------------------
// GLUT̃R[obN֐
//--------------------------------------------------------------
void Mouse::CallBackMove( int x, int y )
{
	
	m_nMoveX = x;
	m_nMoveY = y;
	
	m_bMove = true;
	//glutWarpPointer(20,20);
	//printf("Mouse Move  x:%d  y:%d\n", x, y );
}

//--------------------------------------------------------------
// 
//--------------------------------------------------------------
int Mouse::OnInit()
{
	// glutR[obNo^
	glutMotionFunc( dragMouse );
    glutMouseFunc( clickMouse );
	glutPassiveMotionFunc( moveMouse );

	//glutSetCursor(GLUT_CURSOR_BOTTOM_LEFT_CORNER);
	//glutSetCursor(GLUT_CURSOR_NONE);

	return TR_OK;
}

//--------------------------------------------------------------
// ^XNC
//--------------------------------------------------------------
int Mouse::OnMain()
{
	if( m_bMove ) {
		m_bMove = false;
		m_nMoveDX = m_nMoveX - m_nPrevMoveX;
		m_nMoveDY = m_nMoveY - m_nPrevMoveY;
	} else {
		m_nMoveDX = 0;
		m_nMoveDY = 0;
	}

	m_nPrevMoveX = m_nMoveX;
	m_nPrevMoveY = m_nMoveY;
	
	if( m_bDrag ) {
		m_bDrag = false;
		m_nDragDX = m_nX - m_nPrevX;
		m_nDragDY = m_nY - m_nPrevY;
	} else {
		m_nDragDX = 0;
		m_nDragDY = 0;
	}

	if( m_bFPSMode ) {
		int x = sc.nScreenWidth/2;
		int y = sc.nScreenHeight/2;
		glutWarpPointer(x,y);
		m_nMoveX = x;
		m_nMoveY = y;
		m_nPrevMoveX = x;
		m_nPrevMoveY = y;

		m_nX = x;
		m_nY = y;
		m_nPrevX = x;
		m_nPrevY = y;
	}
	return TR_OK;
}

//--------------------------------------------------------------
// 
//--------------------------------------------------------------
int Mouse::OnKill()
{
	return TR_OK;
}

//--------------------------------------------------------------
// NbN擾
//--------------------------------------------------------------
int Mouse::Click( u32 button )
{
	return ( s_pSelf->m_unClick & button);
}

//--------------------------------------------------------------
// hbO擾
//--------------------------------------------------------------
int Mouse::Drag( u32 button )
{
	return ( s_pSelf->m_unDrag & button);
}

//--------------------------------------------------------------
// |Cg擾
//--------------------------------------------------------------
int Mouse::GetPoint( int &x, int &y )
{
	x = s_pSelf->m_nMoveX;
	y = s_pSelf->m_nMoveY;

	return 0;
}

//--------------------------------------------------------------
// hbÖړʎ擾
//--------------------------------------------------------------
int Mouse::GetDrag( int &x, int &y )
{
	x = s_pSelf->m_nDragDX;
	y = s_pSelf->m_nDragDY;

	return 0;
}

//--------------------------------------------------------------
// ړʎ擾
//--------------------------------------------------------------
int Mouse::GetMove( int &x, int &y )
{
	x = s_pSelf->m_nMoveDX;
	y = s_pSelf->m_nMoveDY;

	return 0;
}

//--------------------------------------------------------------
// NbNW擾
//--------------------------------------------------------------
int Mouse::GetClickPoint( int &x, int &y )
{
	x = s_pSelf->m_nClickX;
	y = s_pSelf->m_nClickY;

	return 0;
}
